unit EditFilter;

(************************************************************************)
(* EDITFILTER v 1.0                                                     *)
(* YOU MAY DISTRIBUTED THIS UNIT BUT PLEASE DO NOT REMOVE THIS BANNER   *)
(************************************************************************)
(* COPYRIGHT (c) 2002 - EDHI NUGROHO (edhinug@unisbank.ac.id)           *)
(************************************************************************)
(* FUNCTION : To construct 'where clause' in query by giving a          *)
(*  template.                                                           *)
(* SINTAX : ; --> DELIMITER                                             *)
(*          - --> RANGE (field >= opr1 and field <= opr2)               *)
(*          % --> LIKE ( field like 'opr')                              *)
(* EXAMPLES :                                                           *)
(* 1. To build where clause like                                        *)
(*      (field1 = 'A') or (field1 >= 'C' and field1 <= 'R')             *)
(*    gives input 'A;C-R' and fill property fieldname with 'field1'     *)
(*    and retrieves the where clause with function                      *)
(*    sql_filter:=EditFilter1.AsString                                  *)
(*                                                                      *)
(* 2. To build where clause like                                        *)
(*      (field1 LIKE 'A%')                                              *)
(*    gives input 'A%' and fill property fieldname with 'field1'        *)
(*    and retrieves the where clause by EditFilter1.AsString            *)
(*                                                                      *)
(* If you want to use integer value or float then use                   *)
(* AsInteger or AsFloat                                                 *)
(************************************************************************)
(* If you've found that this unit is useful for you then send me email  *)
(************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

  TEditFilter = class(TEdit)
  private
    { Private declarations }
    fstack:TStringList;
    ffield:string;
    function StackEmpty:Boolean;
    function GetField:string;
    procedure SetField(f:string);
    procedure Push(o:string);
    function Pop:string;
    function BuildQuery(tipe:integer):string;
    procedure BuildStack;
    procedure ClearStack;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Owner:TComponent);override;
    destructor Destroy;override;
    function AsString:string;
    function AsFloat:string;
    function AsInteger:string;
  published
    { Published declarations }
    property FieldName:string read GetField write SetField;
{    Text:string read GetText write SetText;}
  end;

procedure Register;

implementation

const
 PARAM_AS_STRING=0;
 PARAM_AS_INTEGER=1;
 PARAM_AS_FLOAT=2;

procedure Register;
begin
  RegisterComponents('Samples', [TEditFilter]);
end;

{-- private section --}
function TEditFilter.StackEmpty:Boolean;
begin
 result:=(fStack.Count = 0);
end;

function TEditFilter.GetField:string;
begin
 result:=ffield;
end;

procedure TEditFilter.SetField(f:string);
begin
 ffield:=f;
end;

procedure TEditFilter.Push(o:string);
begin
 fstack.Add(o);
end;

function TEditFilter.Pop:string;
begin
if not StackEmpty then
 begin
  result:=fstack.Strings[fstack.Count-1];
  fstack.Delete(fstack.Count-1);
 end;
end;

procedure TEditFilter.ClearStack;
begin
 fStack.Clear;
end;

function TEditFilter.BuildQuery(tipe:integer):string;
function ChangeStr(tipe:integer;param:string):string;
begin
 case tipe of
  PARAM_AS_STRING:
   begin
    result:=QuotedStr(param);
   end;
  PARAM_AS_INTEGER:
   begin
    result:=param;
   end;
  PARAM_AS_FLOAT:
   begin
    result:=QuotedStr(param);
   end;
 end;
end;

var
 opr1,opr2,oprnd:string;
 query_str:string;
begin
 query_str:='';
 while not StackEmpty do
  begin
   oprnd:=Pop;
   if oprnd = '-' then
    begin
     opr1:=Pop;
     opr2:=Pop;
     if (query_str = '') then
      query_str:=Format('(%s >= %s and %s <= %s)',
        [ffield,ChangeStr(tipe,opr1),ffield,ChangeStr(tipe,opr2)])
     else
      query_str:=query_str+Format(' or (%s >= %s and %s <= %s)',
        [ffield,ChangeStr(tipe,opr1),ffield,ChangeStr(tipe,opr2)])
    end
   else if (Pos('%',oprnd)>0) then
    begin
     if (query_str = '') then
      query_str:=Format('(%s like %s )',
        [ffield,ChangeStr(tipe,oprnd)])
     else
      query_str:=query_str+Format(' or (%s like %s )',
        [ffield,ChangeStr(tipe,oprnd)])
    end
   else
    begin
     if (query_str = '') then
      query_str := Format(' (%s = %s) ',[ffield,ChangeStr(tipe,oprnd)])
     else
      query_str := query_str + Format(' or (%s = %s) ',[ffield,ChangeStr(tipe,oprnd)]);
    end;
  end;
 result:=query_str;
end;

procedure TEditFilter.BuildStack;
var
 p,l:integer;
 s:string;
 c:char;
 opr1,opr2,opr:string;
begin
 ClearStack;
 s:=Text;
 l:=length(s);
 p:=1;
 opr1:='';
 opr2:='';
 opr:='';
 while (p <= l) do
  begin
   {--abaikan white space--}
   while (s[p] = '') do
    inc(p);
   opr1:='';
   opr2:='';
   c:=s[p];
   while (not(c in ['-',';'])) and (p <= l) do
    begin
     opr1:=opr1+c;
     inc(p);
     c:=s[p];
    end;
    if (opr1 <> '') then
     push(opr1);
    case c of
     '-' : Begin
            opr:=c;
            inc(p);
            c:=s[p];
            while (not(c in ['-',';'])) and (p <= l) do
             begin
              opr2:=opr2+c;
              inc(p);
              c:=s[p];
             end;
            opr1:=pop;
            push(opr2);
            push(opr1);
            push(opr);
           End;
     ';' : Begin
            inc(p);
           End;
    end;
  end;
end;


{-- public section --}
constructor TEditFilter.Create(Owner:TComponent);
begin
 inherited Create(Owner);
 fstack:=TStringList.Create;
end;

destructor TEditFilter.Destroy;
begin
 fstack.Free;
 inherited Destroy;
end;

function TEditFilter.AsString:string;
begin
 BuildStack;
 result:=BuildQuery(PARAM_AS_STRING);
end;

function TEditFilter.AsFloat:string;
begin
 BuildStack;
 result:=BuildQuery(PARAM_AS_FLOAT);
end;

function TEditFilter.AsInteger:string;
begin
 BuildStack;
 result:=BuildQuery(PARAM_AS_INTEGER);
end;

end.
