//------------------------------------------------------------------------------
//--  Flat Controls                                                           --
//--  copyright Koen Van Baelen 2002                                          --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or modify    --
//--  it under the terms of the GNU General Public License as published by    --
//--  the Free Software Foundation; either version 2 of the License, or (at   --
//--  your option) any later version.                                         --
//--                                                                          --
//--  This program is distributed in the hope that it will be useful, but     --
//--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
//--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        --
//--  General Public License for more details.                                --
//--                                                                          --
//--  You should have received a copy of the GNU General Public License       --
//--  along with this program; if not, write to the Free Software             --
//--  Foundation, Inc., 59 Temple Place, Suite 330, Boston,                   --
//--  MA 02111-1307 USA or download it at www.opensource.org                  --
//------------------------------------------------------------------------------

unit FlatControls;

interface

//------------------------------------------------------------------------------

uses
  Controls,Windows,Classes,Messages,Graphics,Forms;

//------------------------------------------------------------------------------

type
  THAlign = (haLeft,haCenter,haRight);
  TVAlign = (vaTop,vaCenter,vaBottom);
  TFlatButtonStyle = (fbNormal,fbFlat);
  TGlyphStyle = (gsGlyphLeft,gsGlyphRight,gsGlyphTop,gsGlyphBottom);
  TCheckColor = (ccRed,ccBlue,ccYellow,ccGreen,ccPurple);

//------------------------------------------------------------------------------

  TCustomFlatControl = class(TCustomControl)
  private
    MouseOnControl : Boolean;
    FOnMouseEnter  : TNotifyEvent;
    FOnMouseLeave  : TNotifyEvent;
    procedure FocusRect(X1,Y1,X2,Y2 : Integer);
  protected
    procedure MouseEnter(var Msg : TMessage); message CM_MOUSEENTER;
    procedure MouseLeave(var Msg : TMessage); message CM_MOUSELEAVE;
    procedure SetCtrlHasFocus(var Msg : TMessage); message WM_SETFOCUS;
    procedure KillCtrlHasFocus(var Msg : TMessage); message WM_KILLFOCUS;
    procedure Click; override;
  public
    constructor Create(AOwner : TComponent); override;
    property OnMouseEnter : TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave : TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

//------------------------------------------------------------------------------

  TFlatBitmapButton = class(TCustomFlatControl)
  private
    FGlyph       : TBitmap;
    FStyle       : TFlatButtonStyle;
    FDown        : Boolean;
    FGlyphstyle  : TGlyphStyle;
    FCaption     : String;
    FOnClick     : TNotifyEvent;
    FModalResult : TModalResult;
    FHAlign      : THAlign;
    FVAlign      : TVAlign;
    procedure SetGlyph(Value : TBitmap);
    procedure SetCaption(Value : String);
    procedure SetStyle(Value : TFlatButtonStyle);
    procedure SetGlyphStyle(Value : TGlyphStyle);
    procedure SetModalResult(Value : TModalResult);
    procedure SetHAlign(Value : THAlign);
    procedure SetVAlign(Value : TVAlign);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure Paint; override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property Down : Boolean read FDown;
  published
    property Caption     : String read FCaption write SetCaption;
    property Glyph       : TBitmap read FGlyph write SetGlyph;
    property GlyphStyle  : TGlyphStyle read FGlyphStyle write SetGlyphStyle default gsGlyphLeft;
    property ModalResult : TModalResult read FModalResult write SetModalResult default mrNone;
    property Style       : TFlatButtonStyle read FStyle write SetStyle default fbNormal;
    property HAlign      : THAlign read FHAlign write SetHAlign default haCenter;
    property VAlign      : TVAlign read FVAlign write SetVAlign default vaCenter;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;

    property OnClick : TNotifyEvent read FOnClick write FOnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnStartDrag;
  end;

//------------------------------------------------------------------------------

  TFlatSpeedButton = class(TCustomFlatControl)
  private
    FGlyph    : TBitmap;
    FStyle    : TFlatButtonStyle;
    FDown     : Boolean;
    FIsSwitch : Boolean;
    FOnClick  : TNotifyEvent;
    procedure SetGlyph(Value : TBitmap);
    procedure SetIsSwitch(Value : Boolean);
    procedure SetStyle(Value : TFlatButtonStyle);
    procedure SetDown(Value : Boolean);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property Down : Boolean read FDown write SetDown;
  published
    property Glyph    : TBitmap read FGlyph write SetGlyph;
    property IsSwitch : Boolean read FIsSwitch write SetIsSwitch;
    property Style    : TFlatButtonStyle read FStyle write SetStyle;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property HelpContext;
    property ShowHint;
    property Visible;

    property OnClick : TNotifyEvent read FOnClick write FOnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnStartDrag;
  end;

//------------------------------------------------------------------------------

  TRoundButton = class(TCustomFlatControl)
  private
    FGlyph   : TBitmap;
    FStyle   : TFlatButtonStyle;
    FDown    : Boolean;
    FOnClick : TNotifyEvent;
    procedure SetGlyph(Value : TBitmap);
    procedure SetStyle(Value : TFlatButtonStyle);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property Down : Boolean read FDown;
  published
    property Glyph : TBitmap read FGlyph write SetGlyph;
    property Style : TFlatButtonStyle read FStyle write SetStyle;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property HelpContext;
    property ShowHint;
    property Visible;

    property OnClick : TNotifyEvent read FOnClick write FOnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnStartDrag;
  end;

//------------------------------------------------------------------------------

  TFlatCheckBox = class(TCustomFlatControl)
  private
    FCaption    : String;
    FChecked    : Boolean;
    FCheckColor : TCheckColor;
    FOnClick    : TNotifyEvent;
    procedure SetCaption(Value : String);
    procedure SetChecked(Value : Boolean);
    procedure SetCheckColor(Value : TCheckColor);
  protected
    procedure Paint; override;
    procedure Click; override;
    procedure KeyPress(var Key : Char); override;
    procedure DrawCheckMark; virtual;
    procedure DrawBox(Raised : Boolean); virtual;
  public
    constructor Create(AOwner : TComponent); override;
  published
    property Caption    : String read FCaption write SetCaption;
    property Checked    : Boolean read FChecked write SetChecked;
    property CheckColor : TCheckColor read FCheckColor write SetCheckColor;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnClick : TNotifyEvent read FOnClick write FOnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnStartDrag;
  end;

//------------------------------------------------------------------------------

  TFlatRadioButton = class(TCustomFlatControl)
  private
    FCaption    : String;
    FChecked    : Boolean;
    FCheckColor : TCheckColor;
    FOnClick    : TNotifyEvent;
    procedure SetCaption(Value : String);
    procedure SetChecked(Value : Boolean);
    procedure SetCheckColor(Value : TCheckColor);
  protected
    procedure DrawCheckMark; virtual;
    procedure Click; override;
    procedure Paint; override;
    procedure KeyPress(var Key : Char); override;
  public
    constructor Create(AOwner : TComponent); override;
  published
    property Caption    : String read FCaption write SetCaption;
    property Checked    : Boolean read FChecked write SetChecked;
    property CheckColor : TCheckColor read FCheckColor write SetCheckColor;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnClick : TNotifyEvent read FOnClick write FOnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnStartDrag;
  end;

//------------------------------------------------------------------------------

procedure Register;

implementation

//------------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('Flat Controls',[TFlatBitmapButton,TFlatSpeedButton,
  TRoundButton,TFlatCheckBox,TFlatRadioButton]);
end;

//------------------------------------------------------------------------------
//--   TCustomFlatControl class   ----------------------------------------------
//------------------------------------------------------------------------------

constructor TCustomFlatControl.Create(AOwner : TComponent);                     //TCustomFlatControl constructor
begin
  inherited;
  Parent:=AOwner as TWinControl;
  MouseOnControl:=False;                                                        //Mouse is not on control
  Enabled:=True;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.MouseEnter(var Msg : TMessage);                    //React when mouse enters component
begin
  MouseOnControl:=True;                                                         //Mouse is on control
  Msg.Result:=1;
  if Enabled and Assigned(FOnMouseEnter) then FOnMouseEnter(Self);              //Trigger event
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.MouseLeave(var Msg : TMessage);                    //React when mouse leaves
begin
  MouseOnControl:=False;                                                        //Mouse is not on control
  Msg.Result:=1;
  if Enabled and Assigned(FOnMouseLeave) then FOnMouseLeave(Self);              //Trigger event
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.SetCtrlHasFocus(var Msg : TMessage);               //Set control focus
begin
  if (TabStop and Enabled and CanFocus) then SetFocus;                          //Set focus
  Msg.Result:=1;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.KillCtrlHasFocus(var Msg : TMessage);              //Kill control focus
begin
  Msg.Result:=1;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.Click;                                             //Click procedure
begin
  inherited;
  if (Enabled and TabStop and CanFocus) then SetFocus;                          //Set focus when clicked
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TCustomFlatControl.FocusRect(X1,Y1,X2,Y2 : Integer);                  //Draw focus rectangle
var
  Counter : Integer;
begin
  for Counter:=X1 to X2 do if Odd(Counter) then begin                           //Draw horizontal point lines
    Canvas.Pixels[Counter,Y1]:=Canvas.Pen.Color;
    Canvas.Pixels[Counter,Y2]:=Canvas.Pen.Color;
  end;
  for Counter:=Y1 to Y2 do if Odd(Counter) then begin                           //Draw vertical point lines
    Canvas.Pixels[X1,Counter]:=Canvas.Pen.Color;
    Canvas.Pixels[X2,Counter]:=Canvas.Pen.Color;
  end;
end;

//------------------------------------------------------------------------------
//--   TFlatBitmapButton component   -------------------------------------------
//------------------------------------------------------------------------------

constructor TFlatBitmapButton.Create(AOwner : TComponent);                      //TFlatBitmapButton constructor
begin
  inherited;
  Parent:=AOwner as TWinControl;
  Width:=75;                                                                    //Set width
  Height:=25;                                                                   //Set height
  FGlyph:=TBitmap.Create;                                                       //Create glyph
  FDown:=False;                                                                 //Button is not down
  FStyle:=fbNormal;                                                             //Normal style
  FGlyphStyle:=gsGlyphLeft;                                                     //Set glyphstyle
  FCaption:='';                                                                 //No caption
  FHAlign:=haCenter;                                                            //Set horizontal alignment
  FVAlign:=vaCenter;                                                            //Set vertical alignment
  TabStop:=True;                                                                //Control is tabstop
end;

//------------------------------------------------------------------------------

destructor TFlatBitmapButton.Destroy;                                           //TFlatBitmapButton destructor
begin
  FGlyph.Free;                                                                  //Destroy glyph        
  inherited;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.MouseDown(Button: TMouseButton; Shift: TShiftState; //MouseDown procedure
  X, Y: Integer);
begin
  inherited;
  if Enabled then FDown:=True;                                                  //Button is down
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.MouseUp(Button: TMouseButton; Shift: TShiftState;   //MouseUp procedure
  X, Y: Integer);
begin
  inherited;
  if Enabled then begin
    FDown:=False;                                                               //Button is up
    if ((X > 0) and (Y > 0) and (X < Width) and (Y < Height)) then begin        //If cursor on control...
      if Assigned(FOnClick) then FOnClick(Self);                                //Trigger OnClick event...
      (Owner as TForm).ModalResult:=FModalResult;                               //And send ModalResult to parent form
    end;
  end;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.KeyDown(var Key: Word; Shift: TShiftState);         //KeyDown procedure
begin
  inherited;
  if Enabled and ((Key = 13) or (Key = 32))then begin                           //If key is enter or space...
    FDown:=True;                                                                //Button is down
    Invalidate;
  end;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.KeyUp(var Key: Word; Shift: TShiftState);           //KeyUp procedure
begin
  inherited;
  if Enabled and ((Key = 13) or (Key = 32))then begin                           //If key is enter or space...
    FDown:=False;                                                               //Button is up...
    if Assigned(FOnClick) then FOnClick(Self);                                  //Trigger event...
    (Owner as TForm).ModalResult:=FModalResult;                                 //And send ModalResult to parent form
    Invalidate;
  end;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetGlyph(Value : TBitmap);                          //Set Glyph property
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetCaption(Value : String);                         //Set Caption property
begin
  FCaption:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetStyle(Value : TFlatButtonStyle);                 //Set Style property
begin
  FStyle:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetGlyphStyle(Value : TGlyphStyle);                 //Set GlyphStyle property
begin
  FGlyphStyle:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetModalResult(Value : TModalResult);               //Set ModalResult property
begin
  FModalResult:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetHAlign(Value : THAlign);                         //Set HAlign property
begin
  FHAlign:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.SetVAlign(Value : TVAlign);                         //Set VAlign property
begin
  FVAlign:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatBitmapButton.Paint;                                              //Paint procedure (this one is big!)
var
  Source,Dest    : TRect;
  CapLeft,CapTop : Integer;
begin
  inherited;
  CapLeft:=0;
  CapTop:=0;
  with Canvas do begin
    Brush.Color:=Self.Color;                                                    //Set brush color to button color
    Brush.Style:=bsSolid;                                                       //Set solid brush
    FillRect(ClipRect);                                                         //Clear button
    Font:=Self.Font;                                                            //Set font
    if FGlyph = nil then begin                                                  //If no glyph assigned..
      CapLeft:=(Width - TextWidth(FCaption)) div 2;                             //Set caption X-pos
      CapTop:=(Height - TextHeight(FCaption)) div 2;                            //Set caption Y-pos
      if Enabled then begin                                                     //If enabled...
        if MouseOnControl then Font.Color:=clHighLight                          //Mouse on control -> highlight text
        else Font.Color:=clBlack;                                               //Else normal text
        if not FDown then TextOut(CapLeft,CapTop,FCaption)                      //Draw text when up
        else TextOut(CapLeft + 1,CapTop + 1,FCaption);                          //Same when down
      end else begin                                                            //If not enabled...
        Brush.Style:=bsClear;                                                   //Set clear brush
        Font.Color:=clBtnHighLight;                                             //Set font color for shadow
        TextOut(CapLeft + 1,CapTop + 1,FCaption);                               //Draw text shadow
        Font.Color:=clBtnShadow;                                                //Set font color for face
        TextOut(CapLeft,CapTop,FCaption);                                       //Draw text face
      end;
    end else begin                                                              //If glyph assigned...
      case FGlyphStyle of
        gsGlyphLeft : begin                                                     //GlyphStyle is GlyphLeft...
          case FHAlign of
            haLeft : Dest.Left:=8;                                              //Set glyph X-pos for left align
            haCenter : Dest.Left:=(Width - (FGlyph.Width div 2) -               //Set glyph X-pos for center align
              TextWidth(FCaption) - 10) div 2;
            haRight : Dest.Left:=Width - (FGlyph.Width div 2) -                 // Set glyph X-pos for right align
              TextWidth(FCaption) - 16;
          end;
          Dest.Right:=Dest.Left + (FGlyph.Width div 2);                         //Set glyph width
          CapLeft:=Dest.Right + 8;                                              //Set caption X-pos
          case FVAlign of
            vaTop : Dest.Top:=8;                                                //Set glyph Y-pos for top align
            vaCenter : Dest.Top:=(Height - (FGlyph.Height div 2)) div 2;        //Set glyph Y-pos for center align
            vaBottom : Dest.Top:=Height - (FGlyph.Height div 2) - 8;            //Set glyph Y-pos for bottom align
          end;
          Dest.Bottom:=Dest.Top + (FGlyph.Height div 2);                        //Set glyph height
          CapTop:=Dest.Top + (((FGlyph.Height div 2) - TextHeight(FCaption))    //Set caption Y-pos
            div 2);
        end;
        gsGlyphRight : begin                                                    //Glyph style is GlyphRight...
          case FHAlign of
            haLeft : Dest.Left:=TextWidth(FCaption) + 16;                       //Set glyph X-pos for left alignment
            haCenter : Dest.Left:=(Width - (FGlyph.Width div 2) +               //Set glyph X-pos for center alignment
              TextWidth(FCaption) + 10) div 2;
            haRight : Dest.Left:=Width - (FGlyph.Width div 2) - 8;              //Set glyph X-pos for right alignment
          end;
          Dest.Right:=Dest.Left + (FGlyph.Width div 2);                         //Set glyph width
          CapLeft:=Dest.Left - TextWidth(FCaption) - 8;                         //Set caption X-pos
          case FVAlign of
            vaTop : Dest.Top:=8;                                                //Set glyph Y-pos for top alignment
            vaCenter : Dest.Top:=(Height - (FGlyph.Height div 2)) div 2;        //Set glyph Y-pos for center alignment
            vaBottom : Dest.Top:=Height - (FGlyph.Height div 2) - 8;            //Set glyph Y-pos for bottom alignment
          end;
          Dest.Bottom:=Dest.Top + (FGlyph.Height div 2);                        //Set glyph height
          CapTop:=Dest.Top + (((FGlyph.Height div 2) - TextHeight(FCaption))    //Set caption Y-pos
            div 2);
        end;
        gsGlyphTop : begin                                                      //Glyph style is GlyphTop...
          case FHAlign of
            haLeft : Dest.Left:=8 + ((TextWidth(FCaption) -                     //Set glyph X-pos for left alignment
              (FGlyph.Width div 2)) div 2);
            haCenter : Dest.Left:=(Width - (FGlyph.Width div 2)) div 2;         //Set glyph X-pos for center alignment
            haRight : Dest.Left:=Width - ((TextWidth(FCaption) -                //Set glyph X-pos for right alignment
              (FGlyph.Width div 2)) div 2) - (FGlyph.Width div 2) - 8;
          end;
          Dest.Right:=Dest.Left + (FGlyph.Width div 2);                         //Set glyph width
          CapLeft:=Dest.Left - ((TextWidth(FCaption) - (FGlyph.Width div 2))    //Set caption X-pos
            div 2);
          case FVAlign of
            vaTop : Dest.Top:=8;                                                //Set glyph Y-pos for top align
            vaCenter : Dest.Top:=(Height - (FGlyph.Height div 2) - 16 -         //Set glyph Y-pos for center align
              TextHeight(FCaption)) div 2;
            vaBottom : Dest.Top:=Height - (FGlyph.Height div 2) - 16 -          //Set glyph Y-pos for bottom align
              TextHeight(FCaption);
          end;
          Dest.Bottom:=Dest.Top + (FGlyph.Height div 2);                        //Set glyph height
          CapTop:=Dest.Bottom + 8;                                              //Set caption Y-pos      
        end;
        gsGlyphBottom : begin                                                   //Glyph style is GlyphBottom...
          case FHAlign of
            haLeft : Dest.Left:=8 + ((TextWidth(FCaption) -                     //Set glyph X-pos for left alignment
              (FGlyph.Width div 2)) div 2);
            haCenter : Dest.Left:=(Width - (FGlyph.Width div 2)) div 2;         //Set glyph X-pos for center alignment
            haRight : Dest.Left:=Width - ((TextWidth(FCaption) -                //Set glyph X-pos for right alignment
              (FGlyph.Width div 2)) div 2) - (FGlyph.Width div 2) - 8;
          end;
          Dest.Right:=Dest.Left + (FGlyph.Width div 2);                         //Set glyph width
          CapLeft:=Dest.Left - ((TextWidth(FCaption) - (FGlyph.Width div 2))    //Set caption X-pos
            div 2);
          case FVAlign of
            vaTop : Dest.Top:=16 + TextHeight(FCaption);                        //Set glyph Y-pos for top alignment
            vaCenter : Dest.Top:=(Height - (TextHeight(FCaption) -              //Set glyph Y-pos for center alignment
              (FGlyph.Height div 2) + 8)) div 2;
            vaBottom : Dest.Top:=Height - (FGlyph.Height div 2) - 8;            //Set glyph Y-pos for bottom alignment
          end;
          Dest.Bottom:=Dest.Top + (FGlyph.Height div 2);                        //Set glyph height
          CapTop:=Dest.Top - TextHeight(FCaption) - 8;                          //Set caption Y-pos     
        end;
      end;
      if Enabled then begin                                                     //If control is enabled...
        if not (Focused or MouseOnControl) then begin                           //Not focused or mouse on control...
          if FStyle = fbNormal then begin                                       //Style is normal...
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            MoveTo(0,Height);                                                   //Go to lower left color
            LineTo(0,0);                                                        //Draw line to upper left corner
            LineTo(Width - 1,0);                                                //Draw line to upper right corner
            Pen.Color:=clBtnShadow;                                             //Set pen color to BtnShadow
            LineTo(Width - 1,Height - 1);                                       //Draw line to lower right corner
            LineTo(-1,Height - 1);                                              //Draw line to lower left corner
          end;
          Source.Left:=0;                                                       //Set coordinates for upper left quarter of glyph
          Source.Top:=0;
          Source.Right:=FGlyph.Width div 2;
          Source.Bottom:=FGlyph.Height div 2;
          BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);              //Copy glyph to button
          Font.Color:=Self.Font.Color;                                          //Set font color to default color
        end;
        if MouseOnControl and (not Focused) then begin                          //Mouse on control and not focused...
          Source.Left:=FGlyph.Width div 2;                                      //Set coordinates for upper right quarter of glyph
          Source.Top:=0;
          Source.Right:=FGlyph.Width;
          Source.Bottom:=FGlyph.Height div 2;
          if FStyle = fbNormal then begin                                       //Style is normal...
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            MoveTo(0,Height);                                                   //Go to lower left corner
            LineTo(0,0);                                                        //Draw line to upper left corner
            LineTo(Width - 2,0);                                                //Draw line to upper right corner
            Pen.Color:=clBtnShadow;                                             //Set pen color to BtnShadow
            LineTo(Width - 2,Height - 2);                                       //Draw line to lower right corner
            LineTo(-1,Height - 2);                                              //Draw line to lower left corner
            MoveTo(0,Height - 1);                                               //Go to lower left corner
            Pen.Color:=clBlack;                                                 //Set pen color to black
            LineTo(Width - 1,Height - 1);                                       //Draw line to lower right corner
            LineTo(Width - 1,-1);                                               //Draw line to upper right corner
          end;
          if FStyle = fbFlat then begin                                         //Style is flat...
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            MoveTo(0,Height);                                                   //Go to lower left corner
            LineTo(0,0);                                                        //Draw line to upper left corner
            LineTo(Width - 1,0);                                                //Draw line to upper right corner
            Pen.Color:=clBtnShadow;                                             //Set pen color to BtnShadow
            LineTo(Width - 1,Height - 1);                                       //Draw line to lower right corner
            LineTo(-1,Height - 1);                                              //Draw line to lower left corner
          end;
          BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);              //Copy glyph to button
          Font.Color:=clHighLight;                                              //Set font color to Highlight
        end;
        if Focused and (not FDown) then begin                                   //If focused and not down...
          Source.Left:=FGlyph.Width div 2;                                      //Set coordinates for upper right quarter of glyph
          Source.Top:=0;
          Source.Right:=FGlyph.Width;
          Source.Bottom:=FGlyph.Height div 2;
          if FStyle = fbNormal then begin                                       //Style is normal...
            Pen.Color:=clBlack;                                                 //Set pen color to black
            Rectangle(0,0,Width,Height);                                        //Draw rectangle around button
            MoveTo(0,Height - 2);                                               //Go to lower left corner
            LineTo(Width - 2,Height - 2);                                       //Draw line to lower right corner
            LineTo(Width - 2,-1);                                               //Draw line to upper right corner
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            MoveTo(Width - 3,1);                                                //Go to upper right corner
            LineTo(1,1);                                                        //Draw line to upper left corner
            LineTo(1,Height - 3);                                               //Draw line to lower left corner
            MoveTo(2,Height - 3);                                               //Go to lower left corner
            Pen.Color:=clBtnShadow;                                             //Set color to BtnShadow
            LineTo(Width - 3,Height - 3);                                       //Draw line to lower right corner
            LineTo(Width - 3,1);                                                //Draw line to upper left corner
            Pen.Color:=clHighLight;                                             //Set pen color to Highlight
            BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);            //Copy glyph to button
            FocusRect(4,4,Width - 6,Height - 6);                                //Draw focus rectangle
          end;
          if FStyle = fbFlat then begin                                         //Style is flat...
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            MoveTo(0,Height);                                                   //Go to lower left corner
            LineTo(0,0);                                                        //Draw line to upper left corner
            LineTo(Width - 1,0);                                                //Draw line to upper right corner
            Pen.Color:=clBtnShadow;                                             //Set pen color to BtnShadow
            LineTo(Width - 1,Height - 1);                                       //Draw line to lower right corner
            LineTo(-1,Height - 1);                                              //Draw line to lower left corner
            Pen.Color:=clHighLight;                                             //Set pen color toHighlight
            BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);            //Copy glyph to button
            FocusRect(4,4,Width - 5,Height - 5);                                //Draw focus rectangle
          end;
          Font.Color:=clHighLight;                                              //Set font color to Highlight    
        end;
        if FDown then begin                                                     //Button is down...
          Source.Left:=0;                                                       //Set coordinates for lower left quarter of glyph
          Source.Top:=FGlyph.Height div 2;
          Source.Right:=FGlyph.Width div 2;
          Source.Bottom:=FGlyph.Height;
          Inc(Dest.Left);                                                       //Move glyph and caption 1 pixel to left and down
          Inc(Dest.Top);
          Inc(Dest.Bottom);
          Inc(Dest.Right);
          Inc(CapLeft);
          Inc(CapTop);
          if FStyle = fbNormal then begin                                       //Style is normal...
            if not Focused then begin                                           //Button is not focused...
              Pen.Color:=clBtnShadow;                                           //Set pen color to BtnShadow
              Rectangle(1,1,Width,Height);                                      //Draw rectangle
              Brush.Style:=bsClear;                                             //Set brush style to clear
              Pen.Color:=clBtnHighLight;                                        //Set pen color to BtnHighlight
              Rectangle(0,0,Width - 1,Height - 1);                              //Draw rectangle
              Pen.Color:=clBlack;                                               //Set pen color to black
              Rectangle(0,0,Width,Height);                                      //Draw rectangle
              Pen.Color:=clHighLight;                                           //Set pen color to Highlight
              BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);          //Copy glyph to button
            end else begin                                                      //Button is focused...
              Pen.Color:=clBlack;                                               //Set pen color to black
              Rectangle(0,0,Width,Height);                                      //Draw rectangle
              Pen.Color:=clBtnShadow;                                           //Set pen color to BtnShadow
              Rectangle(1,1,Width - 1,Height - 1);                              //Draw rectangle
              Pen.Color:=clHighLight;                                           //Set pen color to Highlight
              BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);          //Copy glyph to button
              FocusRect(5,5,Width - 5,Height - 5);                              //Draw focus rectangle
            end;
          end;
          if FStyle = fbFlat then begin                                         //Style is flat...
            Pen.Color:=clBtnShadow;                                             //Set pen color to BtnShadow
            MoveTo(0,Height);                                                   //Go to lower left corner
            LineTo(0,0);                                                        //Draw line to upper left corner
            LineTo(Width - 1,0);                                                //Draw line to upper right corner
            Pen.Color:=clBtnHighLight;                                          //Set pen color to BtnHighlight
            LineTo(Width - 1,Height - 1);                                       //Draw line to lower right corner
            LineTo(-1,Height - 1);                                              //Draw line to lower left corner
            Pen.Color:=clHighLight;                                             //Set pen color ro Highlight
            BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);            //Copy glyph to button
            if Focused then FocusRect(5,5,Width - 4,Height - 4);                //Focused -> draw focus rectangle
          end;
          Font.Color:=clHighLight;                                              //Set font color to Highlight
        end;
        TextOut(CapLeft,CapTop,FCaption);                                       //Draw caption
      end else begin                                                            //Button is not enabled...
        if FStyle = fbNormal then begin                                         //Style is normal...
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          MoveTo(0,Height);                                                     //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(-1,Height - 1);                                                //Draw line to lower left corner
        end;
        Source.Left:=FGlyph.Width div 2;                                        //Set coordinates for lower right quarter of glyph
        Source.Top:=FGlyph.Height div 2;
        Source.Right:=FGlyph.Width;
        Source.Bottom:=FGlyph.Height;
        BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);                //Copy glyph to button
        Font.Color:=clBtnHighLight;                                             //Set font color to BtnHighlight
        TextOut(CapLeft + 1,CapTop + 1,FCaption);                               //Draw caption shadow
        Brush.Style:=bsClear;                                                   //Set brush to clear
        Font.Color:=clBtnShadow;                                                //Set font color to Btnshadow
        TextOut(CapLeft,CapTop,FCaption);                                       //Draw caption face
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------
//--   TFlatSpeedButton component   --------------------------------------------
//------------------------------------------------------------------------------

constructor TFlatSpeedButton.Create(AOwner : TComponent);                       //TFlatSpeedButton constructor
begin
  inherited;
  Parent:=AOwner as TWinControl;
  Width:=25;                                                                    //Set width
  Height:=25;                                                                   //Set height
  TabStop:=False;
  FGlyph:=TBitmap.Create;                                                       //Create glyph
  FIsSwitch:=False;                                                             //Not a switch
  FStyle:=fbNormal;                                                             //Style is normal
  FDown:=False;                                                                 //Not down
end;

//------------------------------------------------------------------------------

destructor TFlatSpeedButton.Destroy;                                            //TFlatSpeedButton destructor
begin
  FGlyph.Free;                                                                  //Delete glyph
  inherited;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState;  //MouseDown procedure
  X, Y: Integer);
begin
  inherited;
  if Enabled and (not FIsSwitch) then FDown:=True;                              //No switch -> down
  if Enabled and FIsSwitch then FDown:=not FDown;                               //Switch -> Toggle
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;    //MouseUp procedure
  X, Y: Integer);
begin
  inherited;
  if Enabled and (not FIsSwitch) then FDown:=False;                             //No switch -> up
  if Enabled and Assigned(FOnClick) then FOnClick(Self);                        //Trigger OnClick event
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.SetGlyph(Value : TBitmap);                           //SetGlyph procedure
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.SetDown(Value : Boolean);                            //SetDown procedure
begin
  FDown:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.SetStyle(Value : TFlatButtonStyle);                  //SetStyle procedure
begin
  FStyle:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.SetIsSwitch(Value : Boolean);                        //SetIsSwitch procedure
begin
  FIsSwitch:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatSpeedButton.Paint;                                               //Paint procedure                                   
var
  Source,Dest : TRect;
begin
  inherited;
  with Canvas do begin
    Brush.Color:=Self.Color;                                                    //Set brush color
    Brush.Style:=bsSolid;                                                       //Set brush color to solid
    FillRect(ClipRect);                                                         //Clear button
    with Dest do begin                                                          //Set destination rectangle
      Left:=(Width - (FGlyph.Width) div 2) div 2;
      Top:=(Height - (FGlyph.Height) div 2) div 2;
      Right:=Left + (FGlyph.Width div 2);
      Bottom:=Top + (FGlyph.Height div 2);
    end;
    if Enabled then begin                                                       //If button is enabled...
      if not MouseOnControl then begin                                          //If mouse is not on control...
        if FStyle = fbNormal then begin                                         //If style is normal...
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          MoveTo(0,Height - 1);                                                 //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          MoveTo(Width - 1,1);                                                  //Move a bit
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(0,Height - 1);                                                 //Draw line to lower left corner
        end;
        with Source do begin                                                    //Set coordinates for upper left quarter of glyph
          Left:=0;
          Top:=0;
          Right:=Left + (FGlyph.Width div 2);
          Bottom:=Top + (FGlyph.Height div 2);
        end;
      end;
      if MouseOnControl and (not FDown) then begin                              //Mouse is on control and button is not down...
        if FStyle = fbNormal then begin                                         //If style is normal...
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          MoveTo(0,Height - 1);                                                 //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          Pen.Color:=clBlack;                                                   //Set pen color to black
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(-1,Height - 1);                                                //Draw line to lower left corner
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          MoveTo(1,Height - 2);                                                 //Move a bit
          LineTo(Width - 2,Height - 2);                                         //Draw line to lower right corner
          LineTo(Width - 2,0);                                                  //Draw line to upper right corner
        end;
        if FStyle = fbFlat then begin                                           //If style is flat...
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          MoveTo(0,Height - 1);                                                 //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          MoveTo(Width - 1,1);                                                  //Move a bit
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(0,Height - 1);                                                 //Draw line to lower left corner
        end;
        with Source do begin                                                    //Set coordinates for upper right quarter of glyph
          Left:=FGlyph.Width div 2;
          Top:=0;
          Right:=FGlyph.Width;
          Bottom:=FGlyph.Height div 2;
        end;
      end;
      if FDown and MouseOnControl then begin                                    //Button is down and mouse is on control...
        if FStyle = fbNormal then begin                                         //Style is normal...
          Pen.Color:=clBlack;                                                   //Set pen color to black
          MoveTo(0,Height - 1);                                                 //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(0,Height - 1);                                                 //Draw line to lower left corner
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          MoveTo(1,Height - 2);                                                 //Move a bit
          LineTo(1,1);                                                          //Draw line to upper left corner
          LineTo(Width - 2,1);                                                  //Draw line to upper right corner
        end;
        if FStyle = fbFlat then begin                                           //Style is flat...
          Pen.Color:=clBtnShadow;                                               //Set pen color to BtnShadow
          MoveTo(0,Height - 1);                                                 //Go to lower left corner
          LineTo(0,0);                                                          //Draw line to upper left corner
          LineTo(Width - 1,0);                                                  //Draw line to upper right corner
          MoveTo(Width - 1,1);                                                  //Move a bit
          Pen.Color:=clBtnHighLight;                                            //Set pen color to BtnHighlight
          LineTo(Width - 1,Height - 1);                                         //Draw line to lower right corner
          LineTo(0,Height - 1);                                                 //Draw line to lower left corner
        end;
        with Source do begin                                                    //Set coordinates for lower left quarter of glyph
          Left:=0;
          Top:=FGlyph.Height div 2;
          Right:=FGlyph.Width div 2;
          Bottom:=FGlyph.Height;
        end;
        with Dest do begin                                                      //Move destination rectangle one pixel
          Inc(Left);
          Inc(Top);
          Inc(Right);
          Inc(Bottom);
        end;
      end;
      if FDown and (not MouseOnControl) then begin                              //Button is down but mouse is not on control
        with Source do begin                                                    //Set coordinates for lower left quarter of glyph
          Left:=0;
          Top:=FGlyph.Height div 2;
          Right:=FGlyph.Width div 2;
          Bottom:=FGlyph.Height;
        end;
        with Dest do begin                                                      //Move destination rectangle one pixel
          Inc(Left);
          Inc(Top);
          Inc(Right);
          Inc(Bottom);
        end;
        Pen.Color:=clBtnShadow;                                                 //Set pen color to BtnShadow
        MoveTo(0,Height - 1);                                                   //Go to lower left corner
        LineTo(0,0);                                                            //Draw line to upper left corner
        LineTo(Width - 1,0);                                                    //Draw line to upper right corner
        MoveTo(Width - 1,1);                                                    //Move a bit
        Pen.Color:=clBtnHighLight;                                              //Set pen color to BtnHighlight
        LineTo(Width - 1,Height - 1);                                           //Draw line to lower right corner
        LineTo(0,Height - 1);                                                   //Draw line to lower left corner
      end;
    end else begin                                                              //Button is disabled...
      if FStyle = fbNormal then begin                                           //Style is normal...
        Pen.Color:=clBtnHighLight;                                              //Set pen color to BtnHighlight
        MoveTo(0,Height - 1);                                                   //Go to lower left corner
        LineTo(0,0);                                                            //Draw line to upper left corner
        LineTo(Width - 1,0);                                                    //Draw line to upper right corner
        MoveTo(Width - 1,1);                                                    //Move a bit
        Pen.Color:=clBtnShadow;                                                 //Set pen color to BtnShadow
        LineTo(Width - 1,Height - 1);                                           //Draw line to lower right color
        LineTo(0,Height - 1);                                                   //Draw line to lower left corner
      end;
      with Source do begin                                                      //Set coordinates for lower right quarter of glyph
        Left:=FGlyph.Width div 2;
        Top:=FGlyph.Height div 2;
        Right:=FGlyph.Width;
        Bottom:=FGlyph.Height;
      end;
    end;
    BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);                    //Draw glyph
  end;
end;

//------------------------------------------------------------------------------
//--   TRoundButton component   ------------------------------------------------
//------------------------------------------------------------------------------

constructor TRoundButton.Create(AOwner : TComponent);
begin
  inherited;
  Parent:=AOwner as TWinControl;
  FGlyph:=TBitmap.Create;
  FStyle:=fbNormal;
  FDown:=False;
  TabStop:=False;
  Width:=26;
  Height:=26;
end;

//------------------------------------------------------------------------------

destructor TRoundButton.Destroy;
begin
  FGlyph.Free;
  inherited;
end;

//------------------------------------------------------------------------------

procedure TRoundButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if Enabled then FDown:=True;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TRoundButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if Enabled then FDown:=False;
  if Enabled and (X > 0) and (Y > 0) and (X < (Width - 1))
  and (Y < (Height - 1)) and (Assigned(FOnClick)) then FOnClick(Self);
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TRoundButton.SetGlyph(Value : TBitmap);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TRoundButton.SetStyle(Value : TFlatButtonStyle);
begin
  FStyle:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TRoundButton.Paint;
var
  Source,Dest : TRect;
begin
  inherited;
  with Canvas do begin
    with Dest do begin
      Left:=(Width - (FGlyph.Width div 2)) div 2;
      Top:=(Height - (FGlyph.Height div 2)) div 2;
      Right:=Left + (FGlyph.Width div 2);
      Bottom:=Top + (FGlyph.Height div 2);
    end;
    Brush.Color:=Self.Color;
    FillRect(ClipRect);
    if Enabled then begin
      if not MouseOnControl then begin
        with Source do begin
          Left:=0;
          Top:=0;
          Right:=FGlyph.Width div 2;
          Bottom:=FGlyph.Height div 2;
        end;
        if FStyle = fbNormal then begin
          Brush.Color:=Self.Color;
          Pen.Color:=clBtnShadow;
          Ellipse(1,1,Width - 2,Height - 2);
          Pen.Color:=clBtnHighLight;
          Arc(1,1,Width - 2,Height - 2,Width - 1,0,0,Height - 1);
        end;
      end;
      if MouseOnControl and (not FDown) then begin
        with Source do begin
          Left:=FGlyph.Width div 2;
          Top:=0;
          Right:=FGlyph.Width;
          Bottom:=FGlyph.Height div 2;
        end;
        if FStyle = fbNormal then begin
          Pen.Color:=clBtnShadow;
          Brush.Color:=clBtnShadow;
          Ellipse(0,0,Width - 1,Height - 1);
          Pen.Color:=clBtnHighLight;
          Brush.Color:=clBtnHighLight;
          Pie(0,0,Width - 1,Height - 1,Width - 1,0,0,Height - 1);
          Pen.Color:=clBlack;
          Brush.Style:=bsClear;
          Ellipse(0,0,Width - 1,Height - 1);
          Brush.Style:=bsSolid;
          Brush.Color:=Self.Color;
          Pen.Color:=Self.Color;
          Ellipse(2,2,Width - 3,Height - 3);
        end;
        if FStyle = fbFlat then begin
          Brush.Color:=Self.Color;
          Pen.Color:=clBtnShadow;
          Ellipse(0,0,Width - 1,Height - 1);
          Pen.Color:=clBtnHighLight;
          Arc(0,0,Width - 1,Height - 1,Width - 1,0,0,Height - 1);
        end;
      end;
      if FDown and MouseOnControl then begin
        with Source do begin
          Left:=0;
          Top:=FGlyph.Height div 2;
          Right:=FGlyph.Width div 2;
          Bottom:=FGlyph.Height;
        end;
        with Dest do begin
          Inc(Left);
          Inc(Top);
          Inc(Right);
          Inc(Bottom);
        end;
        if FStyle = fbNormal then begin
          Pen.Color:=clBtnShadow;
          Brush.Color:=clBtnShadow;
          Ellipse(0,0,Width - 1,Height - 1);
          Pen.Color:=clBtnHighLight;
          Brush.Color:=clBtnHighLight;
          Pie(0,0,Width - 1,Height - 1,0,Height - 1,Width - 1,0);
          Pen.Color:=clBlack;
          Brush.Style:=bsClear;
          Ellipse(0,0,Width - 1,Height - 1);
          Brush.Style:=bsSolid;
          Brush.Color:=Self.Color;
          Pen.Color:=Self.Color;
          Ellipse(2,2,Width - 3,Height - 3);
        end;
        if FStyle = fbFlat then begin
          Brush.Color:=Self.Color;
          Pen.Color:=clBtnHighLight;
          Ellipse(0,0,Width - 1,Height - 1);
          Pen.Color:=clBtnShadow;
          Arc(0,0,Width - 1,Height - 1,Width - 1,0,0,Height - 1);
        end;
      end;
    end else begin
      with Source do begin
        Left:=FGlyph.Width div 2;
        Top:=FGlyph.Height div 2;
        Right:=FGlyph.Width;
        Bottom:=FGlyph.Height;
      end;
      if FStyle = fbNormal then begin
        Brush.Color:=Self.Color;
        Pen.Color:=clBtnShadow;
        Ellipse(1,1,Width - 2,Height - 2);
        Pen.Color:=clBtnHighLight;
        Arc(1,1,Width - 2,Height - 2,Width - 1,0,0,Height - 1);
      end;
    end;
    BrushCopy(Dest,FGlyph,Source,FGlyph.Canvas.Pixels[0,0]);
  end;
end;

//------------------------------------------------------------------------------
//--   TFlatCheckBox component   -----------------------------------------------
//------------------------------------------------------------------------------

constructor TFlatCheckBox.Create(AOwner : TComponent);
begin
  inherited;
  Parent:=AOwner as TWinControl;
  FCaption:='';
  FChecked:=False;
  FCheckColor:=ccBlue;
  TabStop:=True;
  Width:=100;
  Height:=20;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.SetCaption(Value : String);
begin
  FCaption:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.SetChecked(Value : Boolean);
begin
  FChecked:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.SetCheckColor(Value : TCheckColor);
begin
  FCheckColor:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.Click;
begin
  inherited;
  if Enabled then begin
    FChecked:=not FChecked;
    SetFocus;
    if Assigned(FOnClick) then FOnClick(Self);
  end;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.KeyPress(var Key : Char);
begin
  inherited;
  if Enabled and ((Key = #13) or (Key = #32)) then begin
    FChecked:=not FChecked;
    if Assigned(FOnClick) then FOnClick(Self);
  end;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.DrawCheckMark;
var
  X,Y        : Integer;
  Dark,Light : TColor;
begin
  Dark:=clNavy;
  Light:=clBlue;
  X:=3;
  Y:=(Height - 12) div 2;
  case FCheckColor of
    ccRed : begin
      Dark:=clMaroon;
      Light:=clRed;
    end;
    ccBlue : begin
      Dark:=clNavy;
      Light:=clBlue;
    end;
    ccYellow : begin
      Dark:=clOlive;
      Light:=clYellow;
    end;
    ccGreen : begin
      Dark:=clGreen;
      Light:=clLime;
    end;
    ccPurple : begin
      Dark:=clPurple;
      Light:=clFuchsia;
    end;
  end;
  with Canvas do begin
    Pixels[X+2,Y+2]:=Light;   Pixels[X+3,Y+3]:=Light;
    Pixels[X+3,Y+4]:=Light;   Pixels[X+3,Y+5]:=Light;
    Pixels[X+4,Y+5]:=Light;   Pixels[X+4,Y+6]:=Light;
    Pixels[X+5,Y+6]:=Light;   Pixels[X+6,Y+5]:=Light;
    Pixels[X+7,Y+4]:=Light;   Pixels[X+8,Y+3]:=Light;
    Pixels[X+9,Y+2]:=Light;   Pixels[X+10,Y+1]:=Light;
    Pixels[X+11,Y+1]:=Light;  Pixels[X+12,Y]:=Light;
    Pixels[X+13,Y]:=Light;    Pixels[X+14,Y-1]:=Light;
    Pixels[X+2,Y+3]:=Dark;   Pixels[X+3,Y+6]:=Dark;
    Pixels[X+4,Y+7]:=Dark;   Pixels[X+4,Y+8]:=Dark;
    Pixels[X+5,Y+7]:=Dark;   Pixels[X+6,Y+6]:=Dark;
    Pixels[X+7,Y+5]:=Dark;   Pixels[X+8,Y+4]:=Dark;
    Pixels[X+9,Y+3]:=Dark;   Pixels[X+10,Y+2]:=Dark;
    Pixels[X+12,Y+1]:=Dark;  Pixels[X+14,Y]:=Dark;
    Pixels[X+15,Y-1]:=Dark;  Pixels[X+3,Y+2]:=clBtnShadow;
    Pixels[X+4,Y+3]:=clBtnShadow;   Pixels[X+4,Y+4]:=clBtnShadow;
    Pixels[X+5,Y+4]:=clBtnShadow;   Pixels[X+6,Y+4]:=clBtnShadow;
    Pixels[X+5,Y+5]:=clBtnShadow;   Pixels[X+2,Y+4]:=clBtnShadow;
    Pixels[X+3,Y+7]:=clBtnShadow;   Pixels[X+5,Y+8]:=clBtnShadow;
    Pixels[X+6,Y+7]:=clBtnShadow;   Pixels[X+7,Y+6]:=clBtnShadow;
    Pixels[X+8,Y+5]:=clBtnShadow;   Pixels[X+9,Y+4]:=clBtnShadow;
    Pixels[X+10,Y+3]:=clBtnShadow;  Pixels[X+11,Y+2]:=clBtnShadow;
    Pixels[X+13,Y+1]:=clBtnShadow;  Pixels[X+15,Y]:=clBtnShadow;
    Pixels[X+13,Y-1]:=clBtnShadow;  Pixels[X+7,Y+7]:=clBtnShadow;
    Pen.Color:=clBtnShadow;
    MoveTo(X+5,Y+8);           LineTo(X+11,Y+8);
    MoveTo(X+6,Y+9);           LineTo(X+10,Y+9);
    MoveTo(X+7,Y+10);          LineTo(X+9,Y+10);
    MoveTo(X+9,Y+7);           LineTo(X+12,Y+7);
    MoveTo(X+10,Y+6);          LineTo(X+13,Y+6);
    MoveTo(X+11,Y+5);          LineTo(X+14,Y+5);
    MoveTo(X+12,Y+4);          LineTo(X+15,Y+4);
    MoveTo(X+13,Y+3);          LineTo(X+17,Y+3);
    MoveTo(X+14,Y+2);          LineTo(X+19,Y+2);
    MoveTo(X+16,Y+1);          LineTo(X+19,Y+1);
  end;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.DrawBox(Raised : Boolean);
var
  Light,Dark : TColor;
  X,Y           : Integer;
begin
  if Raised then begin
    Light:=clBtnHighLight;
    Dark:=clBtnShadow;
  end else begin
    Light:=clBtnShadow;
    Dark:=clBtnHighLight;
  end;
  with Canvas do begin
    X:=14;
    Y:=(Height - 12) div 2;
    MoveTo(X,Y);
    Pen.Color:=Light;
    LineTo(3,Y);
    LineTo(3,Y+11);
    MoveTo(3,Y+11);
    Pen.Color:=Dark;
    LineTo(14,Y+11);
    LineTo(14,Y-1);
  end;
end;

//------------------------------------------------------------------------------

procedure TFlatCheckBox.Paint;
var
  CapX,CapY   : Integer;
  X1,Y1,X2,Y2 : Integer;
begin
  inherited;
  with Canvas do begin
    Brush.Style:=bsSolid;
    Brush.Color:=Self.Color;
    if Enabled then begin
      if MouseOnControl or Focused then begin
        DrawBox(True);
        Font.Color:=clHighLight;
      end else begin
        DrawBox(False);
        Font.Color:=Self.Font.Color;
      end;
      if FChecked then DrawCheckMark;
      CapX:=25;
      CapY:=(Height - TextHeight(FCaption)) div 2;
      TextOut(CapX,CapY,FCaption);
      if Focused then begin
        X1:=CapX - 2;
        Y1:=CapY - 2;
        X2:=X1 + TextWidth(FCaption) + 4;
        Y2:=Y1 + TextHeight(FCaption) + 4;
        Pen.Color:=clHighLight;
        FocusRect(X1,Y1,X2,Y2);
      end;
    end else begin
      FChecked:=False;
      DrawBox(False);
      CapX:=25;
      CapY:=(Height - TextHeight(FCaption)) div 2;
      Font.Color:=clBtnHighLight;
      TextOut(CapX + 1,CapY + 1,FCaption);
      Brush.Style:=bsClear;
      Font.Color:=clBtnShadow;
      TextOut(CapX,CapY,FCaption);
    end;
  end;
end;

//------------------------------------------------------------------------------
//--   TFlatRadioButton component   --------------------------------------------
//------------------------------------------------------------------------------

constructor TFlatRadioButton.Create(AOwner : TComponent);
begin
  inherited;
  Parent:=AOwner as TWinControl;
  FCaption:='';
  FChecked:=False;
  FCheckColor:=ccBlue;
  TabStop:=True;
  Width:=100;
  Height:=20;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.DrawCheckmark;
begin
  with Canvas do begin
    if (MouseOnControl or Focused) then begin
      case FCheckColor of
        ccRed    : Pen.Color:=clRed;
        ccBlue   : Pen.Color:=clBlue;
        ccYellow : Pen.Color:=clYellow;
        ccGreen  : Pen.Color:=clLime;
        ccPurple : Pen.Color:=clFuchsia;
      end;
    end else begin
      case FCheckColor of
        ccRed    : Pen.Color:=clMaroon;
        ccBlue   : Pen.Color:=clNavy;
        ccYellow : Pen.Color:=clOlive;
        ccGreen  : Pen.Color:=clGreen;
        ccPurple : Pen.Color:=clPurple;
      end;
    end;
    MoveTo(10,((Height - 11) div 2) + 3);
    LineTo(10,((Height - 11) div 2) + 8);
    MoveTo(8,((Height - 11) div 2) + 5);
    LineTo(13,((Height - 11) div 2) + 5);
    MoveTo(9,((Height - 11) div 2) + 4);
    LineTo(12,((Height - 11) div 2) + 4);
    MoveTo(9,((Height - 11) div 2) + 6);
    LineTo(12,((Height - 11) div 2) + 6);
  end;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.Click;
var
  Counter : Integer;
begin
  inherited;
  if Enabled then begin
    with (Parent as TWinControl) do begin
      for Counter:=0 to (ControlCount - 1) do begin
        if Controls[Counter] is TFlatRadioButton
        then (Controls[Counter] as TFlatRadioButton).Checked:=False;
      end;
    end;
    SetFocus;
    FChecked:=True;
    if Assigned(FOnClick) then FOnClick(Self);
  end;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.KeyPress(var Key : Char);
var
  Counter : Integer;
begin
  inherited;
  if Enabled and ((Key = #13) or (Key = #32)) then begin
    with (Parent as TWinControl) do begin
      for Counter:=0 to (ControlCount - 1) do begin
        if Controls[Counter] is TFlatRadioButton
        then (Controls[Counter] as TFlatRadioButton).Checked:=False;
      end;
    end;
    FChecked:=True;
    if Assigned(FOnClick) then FOnClick(Self);
  end;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.SetCaption(Value : String);
begin
  FCaption:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.SetChecked(Value : Boolean);
begin
  FChecked:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.SetCheckColor(Value : TCheckColor);
begin
  FCheckColor:=Value;
  Invalidate;
end;

//------------------------------------------------------------------------------

procedure TFlatRadioButton.Paint;
var
  X1,Y1,X2,Y2 : Integer;
begin
  inherited;
  with Canvas do begin
    Brush.Color:=Self.Color;
    Brush.Style:=bsSolid;
    Font:=Self.Font;
    FillRect(ClipRect);
    MoveTo(10,(Height - 11) div 2);
    if Enabled then begin
      if (MouseOnControl or Focused)
      then Pen.Color:=clBtnHighLight
      else Pen.Color:=clBtnShadow;
    end else Pen.Color:=clBtnShadow;
    LineTo(5,((Height - 11) div 2) + 5);
    LineTo(11,((Height - 11) div 2) + 11);
    MoveTo(11,((Height - 11) div 2) + 9);
    if Enabled then begin
      if (MouseOnControl or Focused)
      then Pen.Color:=clBtnShadow
      else Pen.Color:=clBtnHighLight;
    end else Pen.Color:=clBtnHighLight;
    LineTo(15,((Height - 11) div 2) + 5);
    LineTo(10,(Height - 11) div 2);
    if (Enabled and FChecked) then DrawCheckmark;
    if Focused then begin
      X1:=18;
      Y1:=((Height - TextHeight(FCaption)) div 2) - 2;
      X2:=X1 + TextWidth(FCaption) + 4;
      Y2:=Y1 + TextHeight(FCaption) + 4;
      Pen.Color:=clHighLight;
      FocusRect(X1,Y1,X2,Y2);
    end;
    if Enabled then begin
      if (Focused or MouseOnControl) then Font.Color:=clHighLight;
      TextOut(20,(Height - TextHeight(FCaption)) div 2,FCaption)
    end else begin
      Brush.Style:=bsClear;
      Font.Color:=clBtnHighLight;
      TextOut(21,((Height - TextHeight(FCaption)) div 2) + 1,FCaption);
      Font.Color:=clBtnshadow;
      TextOut(20,(Height - TextHeight(FCaption)) div 2,FCaption);
    end;
  end;
end;

//------------------------------------------------------------------------------

end.
