{ Copyright (C) 1998-2000, written by Shkolnik Mike
  FIDOnet: 2:463/106.14
  E-Mail:  mshkolnik@scalabium.com
           mshkolnik@yahoo.com
  WEB: http://www.scalabium.com
       http://www.geocities.com/mshkolnik
  tel: 380-/44/-552-10-29

  In this unit I described the visual dialog for Dataset's Filter/Where clause property editing.
}
unit SMDBFltr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, DB, IniFiles, Buttons;

type
  TSMFilterMode = (fmFilterDataset, fmExpression);
  TSMFilterOption = (foLoadFrom, foSaveAs);
  TSMFilterOptions = set of TSMFilterOption;

  TSMDBFilterItem = class
    FieldIndex: Integer;
    FieldName: string;
    FieldCaption: string;
  end;

  TSMDBFilterDialog = class(TComponent)
  private
    { Private declarations }
    FCaption: TCaption;
    FDataset: TDataset;
    FAllowedFields: TStrings;

    FFilterMode: TSMFilterMode;
    FExpression: string;
    FOptions: TSMFilterOptions;

    FOnBeforeExecute: TNotifyEvent;
    FOnAfterExecute: TNotifyEvent;
    FOnShow: TNotifyEvent;

    procedure SetAllowedFields(Value: TStrings);
  protected
    { Protected declarations }
    function GetItemCaption(item: TSMDBFilterItem): string; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean;
  published
    { Published declarations }
    property Caption: TCaption read FCaption write FCaption;
    property Dataset: TDataset read FDataset write FDataset;
    property AllowedFields: TStrings read FAllowedFields write SetAllowedFields;

    property FilterMode: TSMFilterMode read FFilterMode write FFilterMode;
    property Expression: string read FExpression write FExpression;

    property Options: TSMFilterOptions read FOptions write FOptions default [foLoadFrom, foSaveAs];

    property OnBeforeExecute: TNotifyEvent read FOnBeforeExecute write FOnBeforeExecute;
    property OnAfterExecute: TNotifyEvent read FOnAfterExecute write FOnAfterExecute;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  TfrmFilterDialog = class(TForm)
    lblFilter: TLabel;
    lbFilter: TListView;
    btnFilterAdd: TButton;
    btnFilterDelete: TButton;
    lblAddFilter: TLabel;
    rbAND: TRadioButton;
    rbOR: TRadioButton;
    cbFilterField: TComboBox;
    cbFilterCondition: TComboBox;
    edFilterValue: TEdit;
    lblFilterValue: TLabel;
    lblFilterCondition: TLabel;
    lblFilterField: TLabel;
    btnOk: TButton;
    btnCancel: TButton;
    bvlButton: TBevel;
    btnClear: TButton;
    btnFilterLoad: TSpeedButton;
    btnFilterSave: TSpeedButton;
    procedure btnFilterAddClick(Sender: TObject);
    procedure btnFilterDeleteClick(Sender: TObject);
    procedure lbFilterChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure cbFilterFieldChange(Sender: TObject);
    procedure btnFilterSaveClick(Sender: TObject);
    procedure btnFilterLoadClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
  private
    { Private declarations }
    fltFile: string;
    procedure AddFilter(boolCondition: Boolean; intField, intCondition: Integer; Value: string);
  public
    { Public declarations }
  end;

procedure Register;

implementation

{$R *.DFM}
//{$R SMDBFltr.Dcr}
uses SMCnst, SMDBFltrFile;

procedure Register;
begin
  RegisterComponents('SMComponents', [TSMDBFilterDialog]);
end;

const
  ASignStr: array[0..9] of ShortString = ('', '', '', '', '', '', '', '', '', '');
  ASign: array[0..9] of ShortString = ('=', '<>',
                                       '<=', '>=',
                                       '<', '>',
                                       'NOT IS NULL', 'IS NULL',
                                       'IN', '');
  ACondition: array[Boolean, Boolean] of ShortString = ((' OR  ', 'OR'),
                                                        (' AND ', 'AND'));

{ TSMDBFilterDialog }
constructor TSMDBFilterDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FCaption := 'Filter setup dialog';
  FDataset := nil;
  FAllowedFields := TStringList.Create;
  FOptions := [foLoadFrom, foSaveAs];
end;

destructor TSMDBFilterDialog.Destroy;
begin
  FAllowedFields.Free;

  inherited Destroy;
end;

procedure TSMDBFilterDialog.SetAllowedFields(Value: TStrings);
begin
  FAllowedFields.Assign(Value);
end;

function TSMDBFilterDialog.GetItemCaption(item: TSMDBFilterItem): string;
begin
  Result := {item.FieldName + '  :  ' + }item.FieldCaption;
end;

function GetIndexBySignStr(str: string): Integer;
var i: Integer;
begin
  Result := 0;
  for i := 0 to High(ASignStr) do
    if ASignStr[i] = str then
    begin
      Result := i;
      break
    end;
end;

function GetIndexByName(str: string; lst: TStrings): Integer;
var i: Integer;
begin
  try
    Result := StrToInt(str); //-1;
    for i := 0 to lst.Count-1 do
      if TSMDBFilterItem(lst.Objects[i]).FieldIndex = Result then
      begin
        Result := i;
        break
      end;
  except
    Result := -1;
    for i := 0 to lst.Count-1 do
      if TSMDBFilterItem(lst.Objects[i]).FieldName = str then
      begin
        Result := i;
        break
      end;
  end;
end;


function TSMDBFilterDialog.Execute: Boolean;

  function GetIndexBySign(str: string): Integer;
  var i: Integer;
  begin
    Result := 0;
    for i := 0 to High(ASign) do
      if ASign[i] = str then
      begin
        Result := i;
        break
      end;
  end;

var i, j, k: Integer;
    item: TSMDBFilterItem;
    str, strFilter, strExtracted, strSign: string;
    boolANDOR: Boolean;
begin
  if Assigned(FOnBeforeExecute) then
    FOnBeforeExecute(Self);

//  Result := False;
  with TfrmFilterDialog.Create(Application) do
    try
      Caption := FCaption;

      {fill the native strings}
      ASignStr[0] := strEqual;
      ASignStr[1] := strNonEqual;
      ASignStr[2] := strNonMore;
      ASignStr[3] := strNonLess;
      ASignStr[4] := strLessThan;
      ASignStr[5] := strLargeThan;
      ASignStr[6] := strExist;
      ASignStr[7] := strNonExist;
      ASignStr[8] := strIn;
      ASignStr[9] := strBetween;

      for i := 0 to 9 do
        cbFilterCondition.Items.Add(ASignStr[i]);

      ACondition[False, True] := strOR;
      rbOR.Caption := strOR;
      ACondition[True, True] := strAND;
      rbAND.Caption := strAND;

      lblFilterField.Caption := strField;
      lbFilter.Columns.Items[1].Caption := strField;
      lblFilterCondition.Caption := strCondition;
      lbFilter.Columns.Items[2].Caption := strCondition;
      lblFilterValue.Caption := strValue;
      lbFilter.Columns.Items[3].Caption := strValue;

      btnFilterAdd.Caption := strAddToList;
      btnFilterDelete.Caption := strDeleteFromList;
      lblAddFilter.Caption := strAddCondition;
      lblFilter.Caption := strSelection;

      {New constans by Arpad Toth}
      btnFilterSave.Hint := strFSaveAs;
      btnFilterLoad.Hint := strFLoadFrom;
      btnClear.Caption  := SBtnClear;

      btnOk.Caption := SBtnOk;
      btnCancel.Caption := SBtnCancel;

      if Assigned(FOnShow) then
        FOnShow(Self);

      {fill the field list}
      for i := 0 to DataSet.FieldCount - 1 do
        if (AllowedFields.Count = 0) or
           (AllowedFields.IndexOf(DataSet.Fields[i].FieldName) > -1) then
        begin
          item := TSMDBFilterItem.Create;
          item.FieldIndex := cbFilterField.Items.Count;
          item.FieldName := DataSet.Fields[i].FieldName;
          item.FieldCaption := DataSet.Fields[i].DisplayLabel;

          cbFilterField.Items.AddObject(GetItemCaption(item), item);
        end;


        {fill the filtered conditions}
        case FilterMode of
          fmFilterDataset: strFilter := DataSet.Filter;
          fmExpression: strFilter := Expression;
        end;

        while (strFilter <> '') do
        begin
          strFilter := Trim(strFilter);
          if (Copy(strFilter, 1, 4) = 'OR  ') then
          begin
            Delete(strFilter, 1, 4);
            boolANDOR := False;
          end
          else
          begin
            if (Copy(strFilter, 1, 4) = 'AND ') then
              Delete(strFilter, 1, 4);
            boolANDOR := True;
          end;

          j := Pos(' AND ', strFilter);
          k := Pos(' OR  ', strFilter);
          if (j = 0) and (k = 0) then
            j := Length(strFilter)+1
          else
            if ((k < j) or (j = 0)) and (k > 0) then
              j := k;
          if j > 0 then
          begin
            str := Copy(strFilter, 1, j-1);
            Delete(strFilter, 1, j);

            {extract a fieldname}
            j := Pos(' ', str);
            strExtracted := Copy(str, 2, j-2);
            Delete(str, 1, j);
            i := GetIndexByName(strExtracted, cbFilterField.Items);

            {extract a condition sign}
            j := Pos(' ', str);
            strSign := Copy(str, 1, j-1);
            str := Copy(str, j+2, Length(str)-j-3);

            AddFilter(boolANDOR, i, GetIndexBySign(strSign), str);
          end
        end;

      btnFilterSave.Enabled := (lbFilter.Items.Count>0);

      j := 0;
      if foLoadFrom in Options then
      begin
        btnFilterLoad.Visible := True;
        Inc(j);
      end;
      if foSaveAs in Options then
      begin
        btnFilterSave.Visible := True;
        Inc(j);
      end
      else
        if foLoadFrom in Options then
          btnFilterLoad.Left := btnFilterSave.Left;
      lblFilter.Width := lblFilter.Width - j*btnFilterLoad.Width;
      if j > 0 then
        lblFilter.Width := lblFilter.Width - 5;


      Result := (ShowModal = mrOk);
      if Result then
      begin
        {fill a filter string}
        strFilter := '';
        for i := 0 to lbFilter.Items.Count-1 do
          strFilter := strFilter + PString(lbFilter.Items[i].Data)^;
        if strFilter <> '' then
          strFilter := ' AND ' + strFilter;

        if (strFilter <> '') then
          strFilter := Copy(strFilter, 6, Length(strFilter)-5);

        case FilterMode of
          fmFilterDataset: begin
                             DataSet.Filtered := False;
                             DataSet.Filter := strFilter;
                             DataSet.Filtered := True;
                           end;
          fmExpression: Expression := strFilter;
        end;

      end;

    finally
      Free
    end;

  if Assigned(FOnAfterExecute) then
    FOnAfterExecute(Self);
end;

procedure TfrmFilterDialog.btnFilterAddClick(Sender: TObject);
begin
  AddFilter(rbAND.Checked, cbFilterField.ItemIndex, cbFilterCondition.ItemIndex, edFilterValue.Text);

  rbAND.Checked := True;
  cbFilterField.ItemIndex := -1;
  cbFilterCondition.ItemIndex := -1;
  edFilterValue.Text := '';
end;

procedure TfrmFilterDialog.btnFilterDeleteClick(Sender: TObject);
begin
  with lbFilter do
    Items.Delete(Selected.Index);
   btnFilterSave.Enabled := (lbFilter.Items.Count>0);
end;

procedure TfrmFilterDialog.lbFilterChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  btnFilterDelete.Enabled := Assigned(lbFilter.Selected);
end;

procedure TfrmFilterDialog.cbFilterFieldChange(Sender: TObject);
begin
  btnFilterAdd.Enabled := (cbFilterField.ItemIndex > -1) and
                          (cbFilterCondition.ItemIndex > -1) and
                          ((edFilterValue.Text <> '') or (cbFilterCondition.ItemIndex in [6, 7]));
  btnFilterSave.Enabled := (lbFilter.Items.Count>0);
end;

procedure TfrmFilterDialog.AddFilter(boolCondition: Boolean; intField, intCondition: Integer; Value: string);
var ListItem: TListItem;
    strFieldName, strCondition, s1, s2: string;
    intPos: Integer;
begin
  strFieldName := TSMDBFilterItem(cbFilterField.Items.Objects[intField]).FieldName;
  strCondition := '(' + strFieldName + ' ' +
                  ASign[intCondition];
  if (lbFilter.Items.Count > 0) then
    strCondition := ACondition[boolCondition, False] + strCondition;

  case intCondition of
    {'NOT IS NULL', 'IS NULL'}
    6, 7: strCondition := strCondition + ')';
    {'IN'}
    8: strCondition := strCondition + ' (' + Value + '))';
    {'BETWEEN'}
    9: begin
         intPos := Pos(' ', Value);
         if intPos > 0 then
         begin
           s1 := Copy(Value, 1, intPos-1);
           s2 := Copy(Value, intPos+1, Length(Value)- intPos);
         end
         else
         begin
           s1 := Value;
           s2 := '" "';
         end;
         strCondition := strCondition + ' >= ' + s1 + ') AND (' + strFieldName + ' <= ' + s2 + ')';
       end;
  else
    strCondition := strCondition + ' ''' + Value + ''')';
  end;

  ListItem := lbFilter.Items.Add;
  ListItem.ImageIndex := intField;
  ListItem.Caption := ACondition[boolCondition, True];
  ListItem.SubItems.Add(cbFilterField.Items[intField]);
  ListItem.SubItems.Add(cbFilterCondition.Items[intCondition]);
  ListItem.SubItems.Add(Value);

  ListItem.Data := TObject(LongInt(NewStr(strCondition)));
end;

procedure TfrmFilterDialog.btnFilterSaveClick(Sender: TObject);
var
  i: Integer;
  s, strIndex: string;
begin
  if frmFilterFileDialog.Execute(fltFile, True) then
    with TIniFile.Create(fltFile) do
      try
        EraseSection('Filter');
        for i := 0 to lbFilter.Items.Count-1 do
        begin
          s := lbFilter.Items[i].Caption;
          if s = strAND then
            s := 'AND'
          else
            s := 'OR';
          strIndex := IntToStr(i);
          WriteString('Filter', 'AndOr' + strIndex, s);
          s := TSMDBFilterItem(cbFilterField.Items.Objects[lbFilter.Items[i].ImageIndex]).FieldName;
          WriteString('Filter', 'Field' + strIndex, s);
          WriteInteger('Filter','Condition' + strIndex, GetIndexBySignStr(lbFilter.Items[i].SubItems[1]));
          WriteString('Filter', 'Value' + strIndex, lbFilter.Items[i].SubItems[2]);
        end;
      finally
        Free;
      end;
end;

procedure TfrmFilterDialog.btnFilterLoadClick(Sender: TObject);
var
  i: Integer;
  lst: TStrings;
  AndOr: Boolean;
begin
  if frmFilterFileDialog.Execute(fltFile, False) then
  begin
    lst := TStringList.Create;
    with TIniFile.Create(fltFile) do
      try
        ReadSection('Filter', lst);
        lbFilter.Items.Clear;
        i := 0;
        while i < (lst.Count-1) do
        begin
          AndOr := (ReadString('Filter', lst[i], '') = 'AND');
          AddFilter(AndOr,
                    GetIndexByName(ReadString('Filter', lst[i+1], ''), cbFilterField.Items),
                    GetIndexBySignStr(ASignStr[ReadInteger('Filter', lst[i+2], 0)]),
                    ReadString('Filter', lst[i+3], ''));
          i := i + 4;
        end;
        lbFilter.Refresh;
      finally
        lst.Free;
        Free;
        btnFilterSave.Enabled := (lbFilter.Items.Count>0);
      end;
  end
end;

procedure TfrmFilterDialog.btnClearClick(Sender: TObject);
begin
  lbFilter.Items.Clear;
end;

end.
