unit UGraphUtils;

interface

const
    DT_ONE_DAY_IN_MINUTES           = 1440;  //24*60
    DT_ONE_HOUR_IN_MINUTES          = 60;
    DT_ONE_DAY_IN_HOURS             = 24;

function DBN_Now : TDateTime;
function DT_IncDays(const aDateTime : TDateTime;
                    const aDays : LongInt) : TDateTime;
function DT_IncMinutes(const aDateTime : TDateTime;
                       const aMinutes       : LongInt) : TDateTime;
function STR_IntToDigitGroupedString(aInt : Real) : String; //real is used for big numbers
function DT_DateTimeDiffInMinutes(const aDateTime1 : TDateTime;
                                  const aDateTime2 : TDateTime) : LongInt;
function DT_IncHours(const aDateTime : TDateTime;
                     const aHours : LongInt) : TDateTime;
function DT_DateTimeDiffInHours(const aDateTime1 : TDateTime;
                                const aDateTime2 : TDateTime) : LongInt;

implementation

uses
  Windows, SysUtils;

function DBN_Now : TDateTime;
var
    aSystemTime : TSystemTime;
begin
	  GetSystemTime(aSystemTime);
 	  Result := SystemTimeToDateTime(aSystemTime);
end;

function DT_IncDays(const aDateTime : TDateTime;
                    const aDays : LongInt) : TDateTime;
var
   aMinutes : LongInt;
begin
     aMinutes := aDays*DT_ONE_DAY_IN_MINUTES;
     Result   := DT_IncMinutes(aDateTime,aMinutes);
end;

function DT_IncMinutes(const aDateTime : TDateTime;
                       const aMinutes       : LongInt) : TDateTime;
begin
     Result := aDateTime + (aMinutes/DT_ONE_DAY_IN_MINUTES);
end;

function STR_IntToDigitGroupedString(aInt : Real) : String; //real is used for big numbers
var
    s : String;
begin
    s := FloatToStr(aInt);
    Result := '';
    while true do begin
        if (length(s) <= 3) then begin
            Result := s + ',' + Result;
            break;
        end;
        Result := Copy(s, length(s)-2, 3) + ',' + Result;
        Delete(s, length(s)-2, 3);
    end;
    Delete(Result, length(Result), 1);
end;

function DT_DateTimeDiffInMinutes(const aDateTime1 : TDateTime;
                                  const aDateTime2 : TDateTime) : LongInt;
var
   aDiff : TDateTime;
begin
     aDiff := (aDateTime1 - aDateTime2);
     Result := Round(aDiff*DT_ONE_DAY_IN_MINUTES);
end;

function DT_IncHours(const aDateTime : TDateTime;
                     const aHours : LongInt) : TDateTime;
var
   aMinutes : LongInt;
begin
     aMinutes := aHours*DT_ONE_HOUR_IN_MINUTES;
     Result   := DT_IncMinutes(aDateTime,aMinutes);
end;

function DT_DateTimeDiffInHours(const aDateTime1 : TDateTime;
                                const aDateTime2 : TDateTime) : LongInt;
var
   aDiff : TDateTime;
begin
     aDiff := (aDateTime1 - aDateTime2);
     Result := Round(aDiff*DT_ONE_DAY_IN_HOURS);
end;

end.
 