{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, 2000-2002 Mats Asplund
----------------------------------------------------------

Component Name: TmRegistry
        Author: Mats Asplund
      Creation: 2002-10-05
       Version: 1.1
   Description: A component that simplifies reading and writing
                data (strings) to Registry.
        Credit:
        E-mail: masprod@telia.com
          Site: http://go.to/mdp
  Legal issues: All rights reserved 2002 by Mats Asplund

Usage:
  This software is provided 'as-is', without any express or
  implied warranty.  In no event will the author be held liable
  for any  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it
  and redistribute it freely, subject to the following
  restrictions:

  1. The origin of this software must not be misrepresented,
     you must not claim that you wrote the original software.
     If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is
     not required.

  2. Altered source versions must be plainly marked as such, and
     must not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.

  4. If you decide to use this software in any of your applications.
     Send me an EMail and tell me about it.

Quick Reference:
  TmRegistry inherits from TComponent.

  Key-Properties:
    RootKey: The root key in Registry.

    Key: The key under rootkey, where values are going to be stored.

    CanCreate: If true, key is created if it not exists.

  Key-Methods:
    RegData(Value, Data: string): Use this procedure to supply the Value/Data-pairs
    that are going to be stored in Registry.
    Note: If you don't care about the value-names, set Value to ''. The
    value-names will then automatically be named: Value1, Value2, Value3...

    WriteToRegistry: Writes the Value/Data-pairs to Registry.

    ReadFromRegistry(var Data: TStringList): Reads all data that is under the
    specified RegistryKey.
    Note: The Data-order will be the same as when supplying it with the RegData
    command.

    ReadValueFromRegistry(Value: string): Reads data under Value.

    ClearValue: Deletes the key from Registry.

--------------------------------------------------------------------------------
}
unit mRegistry;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Registry;

type
  TmRootKey = (rkHKEY_CLASSES_ROOT, rkHKEY_CURRENT_USER, rkHKEY_LOCAL_MACHINE,
    rkHKEY_USERS, rkHKEY_CURRENT_CONFIG);

  TmRegistry = class(TComponent)
  private
    fAbout: string;
    fKey: string;
    fRootKey: TmRootKey;
    fValues: array of array[0..1] of string;
    fCanCreate: Boolean;
    fVCount: Integer;
    Data: TStringList;
    procedure SetAbout(Value: string);
    function GetRootKey: HKEY;
    { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadFromRegistry(var Data: TStringList);
    function ReadValueFromRegistry(Value: string): string;
    procedure RegData(Value, Data: string);
    function WriteToRegistry: boolean;
    function ClearValue: boolean;
    { Private declarations }
  published
    property RootKey: TmRootKey read fRootKey write fRootKey;
    property Key: string read fKey write fKey;
    property CanCreate: Boolean read fCanCreate write fCanCreate;
    property About: string read fAbout write SetAbout;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmRegistry]);
end;

constructor TmRegistry.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Data:= TStringList.Create;
  fRootKey:= rkHKEY_LOCAL_MACHINE;
  fKey:= '\software\mas prod.\';
  fCanCreate:= true;
  fVCount:= 0;
  fAbout:= 'Version 1.0, 2002  Mats Asplund, http://go.to/masdp';
end;

destructor TmRegistry.Destroy;
begin
  Data.Free;
  inherited Destroy;
end;

procedure TmRegistry.SetAbout(Value: string);
begin
  Exit;
end;

function TmRegistry.GetRootKey: HKEY;
begin
  Result:= HKEY_LOCAL_MACHINE;
  case fRootKey of
    rkHKEY_CLASSES_ROOT:
      Result:= HKEY_CLASSES_ROOT;
    rkHKEY_CURRENT_USER:
      Result:= HKEY_CURRENT_USER;
    rkHKEY_LOCAL_MACHINE:
      Result:= HKEY_LOCAL_MACHINE;
    rkHKEY_USERS:
      Result:= HKEY_USERS;
    rkHKEY_CURRENT_CONFIG:
      Result:= HKEY_CURRENT_CONFIG;
  end;
end;

procedure TmRegistry.RegData(Value, Data: string);
begin
  Inc(fVCount);
  SetLength(fValues, fVCount);
  if Value = '' then
    Value:= 'Value' + IntToStr(fVCount);
  fValues[fVCount - 1, 0]:= Value;
  fValues[fVCount - 1, 1]:= Data;
end;

function TmRegistry.WriteToRegistry: Boolean;
var
  fReg: TRegistry;
  n: Integer;
begin
  Result:= false;
  fReg:= TRegistry.Create;
  try
    fReg.RootKey:= GetRootKey;
    if fReg.KeyExists(fKey) then
    begin
      if fReg.OpenKey(fKey, false) then
      begin
        for n:= 0 to fVCount - 1 do
          fReg.WriteString(fValues[n, 0], fValues[n, 1]);
        Result:= true;
      end
    end
    else
    begin
      if fCanCreate then
        if fReg.OpenKey(fKey, true) then
        begin
          for n:= 0 to fVCount - 1 do
            fReg.WriteString(fValues[n, 0], fValues[n, 1]);
          Result:= true;
        end;
    end;
  finally
    fReg.Free;
  end;
end;

procedure TmRegistry.ReadFromRegistry(var Data: TStringList);
var
  fReg: TRegistry;
  fValueNames: TStringList;
  n: Integer;
begin
  fReg:= TRegistry.Create;
  fValueNames:= TStringList.Create;
  try
    fReg.RootKey:= GetRootKey;
    if fReg.OpenKey(fKey, false) then
    begin
      fReg.GetValueNames(fValueNames);
      for n:= 0 to fValueNames.Count - 1 do
      begin
        Data.Add(fReg.ReadString(fValueNames[n]));
      end;
    end
    else
      Data.Add('');
  finally
    fReg.Free;
    fValueNames.Free;
  end;
end;

function TmRegistry.ClearValue: boolean;
var
  fReg: TRegistry;
begin
  fReg:= TRegistry.Create;
  try
    fReg.RootKey:= GetRootKey;
    if fReg.OpenKey(fKey, false) then
      fReg.DeleteKey(fKey);
  finally
    fReg.Free;
  end;
end;

function TmRegistry.ReadValueFromRegistry(Value: string): string;
var
  fReg: TRegistry;
begin
  fReg:= TRegistry.Create;
  try
    fReg.RootKey:= GetRootKey;
    if fReg.OpenKey(fKey, false) then
      Result:= fReg.ReadString(Value)
    else
      Result:= '';
  finally
    fReg.Free;
  end;
end;

end.

