# regware.pl  --  Version 3.2
#
# Perl unit to generate registration codes for
# TRegware 3.2 Component for Delphi
#  2002 - WAK Productions
# Programmed by Winston Kotzan
#
# To use this module, include regware.pl in your unit like this:
#
# require 'regware.pl';
#
# Then to generate a registration code, use the syntax:
#
# $RegCode = &calculate_regcode($LicenseID, $Seed, $RegCodeSize);
#
# $RegCode is the registration code for $LicenseID.
# $LicenseID, $Seed, and $RegCodeSize correspond to their respective
# properties in the TRegware Delphi component.
#

use Math::BigInt;

require 'regconfig.pl'

# Default values for calculate_regcode()
$MinChars = 3;
$MaxChars = 25;


sub calculate_regcode() {
  my($LicenseID, $Seed, $RegCodeSize)= @_;


  # Create error if LicenseName was greater than MaxChars (25),
  # or shorter than MinChars (3)
  if ((length($LicenseID) < $MinChars) or (length($LicenseID) > $MaxChars)) {
    # Make HTML response for failure
    return 0;
  }

  if ($Seed == 0) { return 0; }
  if ($RegCodeSize == 0) { $RegCodeSize = 12 };

  # Takes the last 6 characters (going backwards), converting each
  # character to ASCII and taking the remainder of dividing it into Seed
  $i = 0;
  $CodeStr = "";
  $ReversedName = reverse $LicenseID;
  while (($i <= 6) and ($i < length($ReversedName))) {
    $CodeStr = $CodeStr . ($Seed % ord(substr($ReversedName, $i, 1)));
    $i++;
  }

  # Convert to Hex
  $BigCodeStr = Math::BigInt->new($CodeStr);
  @hexnums=qw( 0 1 2 3 4 5 6 7 8 9 A B C D E F );
  while($BigCodeStr)
  {
    unshift @RegCode,$hexnums[$BigCodeStr%16];
    $BigCodeStr=$BigCodeStr/16;
  }
  $RegCode=join("", @RegCode);

  if (($RegCodeSize > 0) and (length($RegCode) < $RegCodeSize)) {
    # If RegCode is not big enough then fill the front of it with 0's
    $RegCode = reverse $RegCode;
    for ($diff = $RegCodeSize - length($RegCode); $diff > 0; $diff--) {
      $RegCode = $RegCode . "0";
    }
    $RegCode = reverse $RegCode;
  } elsif (($RegCodeSize > 0) and (length($RegCode) > $RegCodeSize)) {
    # Chop off the ending characters if reg code is bigger than RegCodeSize
    $diff = length($RegCode) - $RegCodeSize;
    while ($diff > 0) {
      chop $RegCode;
      $diff--;
    }
  }
  return $RegCode;
}

1; # This thing has to return a 'true value'