{       WAK Productions Presents:                               }
{       TRegware    version 3.22                                }
{       Copyright 2002 - WAK Productions                       }
{       ---------------------------------                       }
{       Programmed by Winston Kotzan                            }
{       Email: wak@wakproductions.com                           }
{       Website URL: http://www.wakproductions.com/             }
{
This program helps developers of shareware include a timebomb/registration for
their program.  The end-user will enter a License ID and a serial number
(like Winzip).  If their registration information is correct, then the
program becomes registered.  TRegware hides data in the Windows registry
to check if program was registered.

If you have any difficulties using this component, suggestions, or compliments
about this component, I would be glad to hear them!  Please send an Email to
wak@wakproductions.com.  The more user responses received, the more TRegware can
be improved to keep it the best shareware security component for Delphi
developers!

IF YOU HAVE A TECHNICAL QUESTION, PLEASE READ FAQ.TXT BEFORE SENDING AN EMAIL.
THANK YOU!
}
(*
Updates
-------
09-26-02 -- Version 3.22
- Fixed a small error with the clock-tamper feature.  The registry data needed
  to be resaved after checking for a clock tampering.  A call to
  SaveRegistryData() has been added to the Loaded() function to correct this.
- Clock-tamper feature now also checks the file creation timestamp on the file
  for added security.  The clock will be considered "tampered" if the system
  time is before the creation time of the program.

09-15-02 -- Version 3.20
- A set of compiler-time options are now available in the unit reg_opts.pas
  It is suggested to configure this file for each of your programs for
  greater security.  Please read the comments in reg_opts.pas for further
  description.
- Added Email property to contain user's email address. See DoRegistration()
  function documentation for details.
- Licensing options (LicenseType, LicenseSource properties) have been moved
  to a special class, TLicenseOptions.  They will now appear under the
  LicenseOptions property. LicenseOptions are only available in the
  professional edition.


08-09-02 -- Version 3.02 beta
- Fixed bug in GetLicenseID(), changed lines reading:
  "Registry.OpenKey('\Soft..." to "Registry.OpenKeyReadOnly('\Soft..."


06-30-02  --  Version 3.0 beta
- FSeed is now encrypted to protect from hackers.  To turn this option off,
  remove the '$DEFINE HIDE_SEED' line from regware2.pas.  Having this option
  on may cause TRegware to not recognize already existing registrations from
  previous versions.
- Windows 2000 / XP registry is now supported.  Information is stored
  under HKEY_CLASSES_ROOT for 2000/XP systems.
- New ban-list features allows you to ban certain users from using
  the next version of your software
- Added 1 second-delay for registration attempts to prevent cracking by
  brute-force.  This option does not have a property to disable it so that
  it is a little more difficult to patch.
- Added spaghetti code in CalculateCode() for the purpose of confusing hackers
- Option to use system-generated license id  (uses Windows system ID or hard drive serial #)
- Option to set site-licenses and single-user licenses.


06-28-01  --  Version 2.1  (no backwards compatibility)
-Fixed bug that caused program to improperly cut down registration codes that
 were longer RegCodeSize, causing the last number in some codes to be incorrect.
-New feature that detects when user sets clock backwards.  If the user tries
 setting his computer's clock to an earlier time, program will consider itself
 'expired'.  When the user puts his clock back within the remaining x days
 of his evaluation, the program will continue to operate as normal.


Quick Reference
---------------
Here is a sumamary of the usage of the variables in TRegInfo:

License -  Name of the user who registered program
Organization - Organization or company of user who registered program
ExpireTime - TDateTime value of when the program will expire
  (decimal portion is ignored)
LastCountDown - Number of days left of evaluation time, recorded when the program
  was last shut down (used to make sure that user does not set clock backwards)
RegVersion - Version of TRegware used for registry (for future expansion)
RegCode - After the program has been registered, this value contains the serial
  number derived from the License value
Exp2 - Site license or single-user license (implemented in version 3.0)
Exp1 - Contains email address of user (implemented in version 4.0)


ExpX - Not in use - provided for future expansion



End-User License Agreement
--------------------------
By using this software, you are agreeing to the following terms:

1. Although this program has many precautions to secure your software from
 "crackers" and illegal registrations, like any security device there is no
 guarantee that it is 100% fullproof.  USE AT YOUR OWN RISK.

2. WAK Productions or any employee will not be held responsible for any
 illegal registrations of your program.

3. If you would like additional security beyond what is provided in TRegware,
  I suggest customizing the source code by changing the algorithm or using
  different compiler directives.  If you do make changes, it would be
  appeciated if you Email them to wak@wakproductions.com.  Thank you.

*)
unit regware2;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics, Registry, ComObj, Dialogs;

{$i reg_opts.pas}

const
  TREGWARE_VERSION='3.20';

type
  TLicenseType=(ltSiteLicense, ltSingleUserLicense);
  TLicenseTypes=set of TLicenseType;

  { Used for specifying how the License value is derived             }
  {                                                                  }
  { lsUserDefined - user defined license name                        }
  { lsWindowsID - uses the ProductID number of the Windows OS        }
  { lsWindowsIDMod - a processed version of the Windows ProductID    }
  { lsHardDriveSN - volume serial number of the hard drive           }
  TLicenseSource=(lsUserDefined, lsWindowsID, lsWindowsIDMod, lsHardDriveSN);

  {$IFDEF PACKED_RECORDS}
  TRegInfo = packed record
    License, Organization, RegCode, Exp1: string[100];
    RegVersion: string[10];
    LastCountDown, Exp2, Exp3: integer;
    ExpireTime, Exp4: TDateTime;
  end;
  {$ELSE}
  TRegInfo = record
    License, Organization, RegCode, Exp1: string[100];
    RegVersion: string[10];
    LastCountDown, Exp2, Exp3: integer;
    ExpireTime, Exp4: TDateTime;
  end;
  {$ENDIF}

  {$IFDEF PRO_VERSION}

  {INDY1 - This comment is only here for WAK Productions reference}

  TLicenseOptions=class(TPersistent)
    private
      FLicenseSource: TLicenseSource;
      FLicenseType: TLicenseType;
    public
      constructor Create;
    published
      {  LicenseSource determines the method used to obtain the TRegware.LicenseID
         value.  If LicenseSource is lsUserDefined, then the value of LicenseID will
         be determined by the value passed to the DoRegistration() function.
         LicenseID will be a null string until the program has been registered.
         If LicenseOptions.LicenseSource is NOT lsUserDefined, then the value of
         LicenseID will automatically be a system-generated value.  See TLicenseSource
         for more information. }
      property LicenseSource: TLicenseSource read FLicenseSource write FLicenseSource;
      { Use this to specify between single-user licenses and site licenses.  If
        set to ltSiteLicense, the TRegware behaves as if LicenseSource is lsUserDefined,
        regardless of the value LicenseSource is set to. }
      property LicenseType: TLicenseType read FLicenseType write FLicenseType;
  end;
  {$ENDIF}  // end PRO_VERSION

  TRegwareII = class(TComponent)
    private
        FBanList: TStringList;
        FCheckTamper: boolean;
        FComponentVersion: string;
        FDays : Integer;
        FLastCountDown: integer;
        FEmail: string;
        FExpireTime: TDateTime;
        FLicense : String;  // Contains the LicenseID (only if LicenseSource is lsUserDefined)
        FLicenseOptions: TLicenseOptions;
        FMaxChars: integer;
        FMinChars: integer;
        FOrganization : String;
        FProgGUID : String;
        FProgVersion: string;
        FRegCode: string;
        FRegCodeSize: integer;
        FSeed: Int64;
        FTimebomb : Boolean;
        FOnClockChange: TNotifyEvent;
        FOnExpire: TNotifyEvent;
        FOnNagScreen: TNotifyEvent;
        FRegVersion: string;

        {INDY2 - This comment is here for WAK Productions reference only }

        procedure AutoInitialize;
        procedure AutoDestroy;
        function GetDaysLeft : Integer;
        function GetExpired: boolean;
        function GetLicenseID: string;
        function GetRegistered : Boolean;
        function GetSeed: Int64;
        function CheckClockTampered: boolean; // Checks if user set clock backwards
        procedure CheckVariablesSet;
        procedure ClearLicenseInfo;
        procedure LoadRegistryValues;
        procedure SaveRegistryValues;
        procedure SetMaxChars(MaxChars: integer);
        procedure SetMinChars(MinChars: integer);
        procedure SetSeed(Value: Int64);

    protected
        function CheckCodeBanned(RegCode: string): boolean;   // Checks if a registration code was banned
        function CheckForWinNT: boolean;  // Returns true if Windows NT/2000/XP
        procedure Expire;
        procedure ShowNag;
        procedure Loaded; override;
        procedure ClockChange;
        procedure SetBanList(Value: TStringList);

        {$IFDEF PRO_VERSION}
        {INDY10 - This note is for WAK Productions use only }
        {$ENDIF}

        procedure WndProc(var Msg: TMessage);  // Receives Windows messages
    public
      { Calculates the registration code based on the LicenseName variable. }
      function CalculateCode(LicenseName: string): string;
      { Checks if the program's evaluation time has expired. True is returned
        if the time has run out, false if the evaluation period is still
        running. }
      function CheckExpired: boolean;
      { Checks the registry to see if the user has registered the program.
        True is returned if the program is registered, false otherwise. }
      function CheckRegistered: boolean;
      { Creates a new instance of TRegwareII }
      constructor Create(AOwner: TComponent); override;
      { Frees associated memory with TRegwareII.  Do not call Destroy directly,
        instead call the Free method }
      destructor Destroy; override;
      { DoRegistration() "registers" the program.  Note that 'dr' has been         }
      { appended to the parameter names to differentiate those variables from      }
      { TRegware's properties.                                                     }
      {                                                                            }
      { LicenseID is a string used to identify the user. It could be the user's    }
      { name, or a random number.                                                  }
      {                                                                            }
      { Organization is optional, it can be the user's company name. Organization  }
      { is ignored if it is a null string.  It is primarily used to help identify  }
      { the owner of the program.                                                  }
      {                                                                            }
      { RegCode is a serial number based on LicenseID.  This is the "registration  }
      { code" used to check that the user has a valid license.  If the user enters }
      { a RegCode that does not match up with "LicenseID + algorithm()", then      }
      { TRegware rejects the user's registration attempt an remain unregistered.   }
      { Use the CalculateCode() method to calculate the registration code for any  }
      { particular LicenseID.                                                      }
      {                                                                            }
      { Email is an optional email address provided by the user.                   }
      {                                                                            }
      {  Note: If LicenseOptions.LicenseSource is not lsUserDefined, then the      }
      {    value drLicenseID parameter is ignored and will be replaced by a        }
      {    system-generated LicenseID, unique to the computer running the program. }
      {    So, unless lsUserDefined is selected, you can pass an empty string as   }
      {    LicenseName.                                                            }
      function DoRegistration(drLicenseID, drOrganization, drRegCode, drEmail: string): boolean; overload;
      { This overloaded version of DoRegistration() does not require an Email      }
      { parameter.  It is primarily for backwards compatibility with older versions}
      { of TRegware.                                                               }
      function DoRegistration(drLicenseID, drOrganization, drRegCode: string): boolean; overload;
      { Resets the registry so that the program becomes unregistered. If the
        Timebomb property is true, then the timebomb is reset }
      procedure SetUnregistered;

      { The number of days left (if unregistered) for the evaluation period.
        This property is equal to -1 if the program is registered or has passed the
        time limit. }
      property DaysLeft: Integer read GetDaysLeft;

      { Email is an optional email address provided by the user. It will be a null }
      { string until the program is registered via the DoRegistration() function.  }
      property Email: string read FEmail;
      { Indicates whether the program has expired. }
      property Expired: boolean read GetExpired;
      { LicenseID is a string used to identify the user. It could be the user's    }
      { name, or a random number. If LicenseOptions.LicenseSource is               }
      { lsUserDefined, then the value of LicenseID will be blank until the program }
      { has been registered.  If LicenseOptions.LicenseSource is NOT lsUserDefined,}
      { then the value of LicenseID will automatically be a system-generated       }
      { LicenseID, unique to the computer running the program.                     }
      property LicenseID: string read GetLicenseID;
      { Organization is optional, it can be the user's company name. Organization  }
      { is ignored if it is a null string.  It is primarily used to help identify  }
      { the owner of the program. This property will be a null string until the    }
      { program has been registered via the DoRegistration() function              }
       property Organization : String read FOrganization;
      { RegCode is a serial number based on LicenseID.  This is the "registration  }
      { code" used to check that the user has a valid license.  If the user enters }
      { a RegCode that does not match up with "LicenseID + algorithm()", then      }
      { TRegware rejects the user's registration attempt an remain unregistered.   }
      { Use the CalculateCode() method to calculate the registration code for any  }
      { particular LicenseID.  The RegCode property will be a null string until the}
      { program has been registered via the DoRegistration() function.             }
      property RegCode : string read FRegCode;
      { Indicates whether the program has been registered. }
      property Registered : Boolean read GetRegistered;
      { Version of TRegware that wrote the already existing license information }
      property RegVersion: string read FRegVersion;

    published
      { Event is sent after the component loads in memory.  This event means that
        tampering has been done with the system's clock.  Difference is how many
        days back the clock has been set. }
      property OnClockChange: TNotifyEvent read FOnClockChange write FOnClockChange;
      { Event sent on startup of program if program has expired }
      property OnExpire: TNotifyEvent read FOnExpire write FOnExpire;
      { Event sent on startup of program, indicating an appropriate time
        to show a nag screen.  See Demo1 for proper use. }
      property OnNagScreen: TNotifyEvent read FOnNagScreen write FOnNagScreen;
      { A list of registration codes to ban - each line should be a registration
        code that you do not want to work }
      property BanList: TStringList read FBanList write SetBanList;
      { Set this property to true if you want TRegware to check for tampering
        to the system's clock.  Useful for detecting if someone sets their
        clock back to avoid a time bomb.  Also, if TRegware does detect tampering,
        the program will report that it has expired. }
      property CheckTamper: boolean read FCheckTamper write FCheckTamper default true;
      property ComponentVersion: string read FComponentVersion;
      { This is the number of days timebomb will run.  It is ignored if the timebomb is already running. }
      property Days : Integer read FDays write FDays default 30;
      { Maximum number of characters that the License name can be }
      property MaxChars: integer read FMaxChars write SetMaxChars default 25;
      { Minimum number of characters that License name can be }
      property MinChars: integer read FMinChars write SetMinChars default 3;
      { The GUID string indicating the registry key where timebomb and registration
        information is stored.  If the programmer forgets to set this property
        to a valid GUID string, then a message box shows up making sure that
        the programmer is reminded. For more information on GUID's, look up TGUID
        in your Delphi online help file. }
      property ProgGUID : String read FProgGUID write FProgGUID;
      { Set this to the version number of your program.  This value will
        appear in notification Emails if ServerOptions.RegNotify is true }
      property ProgVersion: string read FProgVersion write FProgVersion;
      { A number used in generating the registration code.  Changing this number
        will yield different registration codes. If you forget to set this property
        to a value between 1000 and 2^63, then you WILL BE REMINDED when you
        try running the program. }
      property Seed : Int64 read GetSeed write SetSeed default 0;
      { The number of digits you would like the registration code to be.  Setting
        this to 12 will have TRegwareII generate a registration code 12 digits long }
      property RegCodeSize: integer read FRegCodeSize write FRegCodeSize default 12;
      { Set this to true if you want TRegwareII to cause the program to stop functioning
        after a certain number of days.  If this is set to false, the program
        will not expire. }
      property Timebomb : Boolean read FTimebomb write FTimebomb default True;

      {$IFDEF PRO_VERSION}

      { Values to specify how LicenseID will be generated. This will allow you
        to accomodate site licenses/single user licenses and specify TRegware
        to generate system-unique LicenseID's }
      property LicenseOptions: TLicenseOptions read FLicenseOptions write FLicenseOptions;

      {INDY3 - This comment is here for WAK Productions reference only }

      {$ENDIF}
  end;

procedure Register;

implementation

{INDY4 - This comment is here for WAK Productions reference only }

procedure Register;
begin
     RegisterComponents('WAK', [TRegwareII]);
end;

function FileTimeToDateTime(FileTime: TFileTime): TDateTime;
var
  ModifiedTime: TFileTime;
  SystemTime: TSystemTime;
begin
  FileTimeToLocalFileTime(FileTime, ModifiedTime);
  FileTimeToSystemTime(ModifiedTime, SystemTime);
  Result := SystemTimeToDateTime(SystemTime);
end;

{ Checks for a valid GUID string }

function ValidateGUID(GUIDStr: string): boolean;
  { Check for hex character }
  function IsHex(AlphaNumChar: char): boolean;
  begin
    if ((Ord(AlphaNumChar) >= 48) and (Ord(AlphaNumChar) <= 57)) or
       ((Ord(AlphaNumChar) >= 65) and (Ord(AlphaNumChar) <= 70)) then
       Result := true
    else
      Result := false;
  end;

var
  i: integer;
begin
  Result := false;
  if Length(GUIDStr) <> 38 then Exit; // Not proper length
  for i := 1 to Length(GUIDStr) do
  begin
    // Check for valid letters and numbers
    if (i <> 1) and (i <> 10) and (i <> 15) and (i <> 20) and (i <> 25) and (i <> 38) then
    begin
      if not IsHex(GUIDStr[i]) then Exit;
    end else
      // Check for valid brackets
      if (i = 1) and (GUIDStr[i] <> '{') then Exit
        else if (i = 38) and (GUIDStr[i] <> '}') then Exit
          // Hyphens
          else if (i = 10) or (i = 15) or (i = 20) or (i = 25) then
            if (GUIDStr[i] <> '-') then Exit;
  end;
  Result := true;  // Ok, if all the above conditions checked;
end;

{------------------ TLicenseOptions/TServerOptions methods ------------------}

{$IFDEF PRO_VERSION}

constructor TLicenseOptions.Create;
begin
  inherited Create;
  FLicenseSource := lsUserDefined;
  FLicenseType := ltSingleUserLicense;
end;

{INDY5 - This comment is here for WAK Productions reference only}

{$ENDIF}

{------------------ TRegwareII methods ------------------}


{$IFDEF PRO_VERSION}

{------------------ TRegwareII Pro-version only functions ------------------}

{INDY6 - This comment is here for WAK Productions reference only}

{$ENDIF}  // end pro version

{------------------ TRegwareII methods (standard version) ------------------}

{ Method to set variable and property values and create objects }
procedure TRegwareII.AutoInitialize;
begin
     FCheckTamper := true;
     FDays := 30;
     FMaxChars := 25;
     FMinChars := 3;
     FTimebomb := True;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }
procedure TRegwareII.AutoDestroy;
begin
end;

function TRegwareII.GetDaysLeft : Integer;
begin
  if CheckClockTampered then // User tampered with clock
    Result := 0   // Consider program as expired
  else if (not Registered) and (Timebomb = true) then
  begin
    Result := Trunc(FExpireTime - Date);
    if Result < 0 then Result := 0;
  end
  else  // Clock not tampered and program registered
    Result := -1;
end;

function TRegwareII.GetLicenseID: string;

  function GetWindowsID: string;
  var
    Registry: TRegistry;
  begin
    Registry := TRegistry.Create(KEY_WRITE);
    try
      Registry.Lazywrite := false;
      Registry.RootKey := HKEY_LOCAL_MACHINE;
      if CheckForWinNT = true then
        Registry.OpenKeyReadOnly('\Software\Microsoft\Windows NT\CurrentVersion')
      else
        Registry.OpenKeyReadOnly('\Software\Microsoft\Windows\CurrentVersion');
      Result := Registry.ReadString('ProductID');
      Registry.CloseKey;
    finally
      Registry.Free;
    end; // try..finally
  end;

  function EncryptID(ID: string): string;
  const
    CRYPT='1AZQS2XWCDXXXXXXX3E4RFVTBG56HYNJU7MKI8O9L0P';
  var
    i: integer;
  begin
    for i := 1 to Length(ID) do
    begin
      if ID[i] = '-' then
        Result := Result + '-'
      else
        Result := Result + CRYPT[Ord(ID[i]) - 47];
    end;
  end;

  function GetHardDiskSerial(const DriveLetter: Char): string; 
  var 
    NotUsed:     DWORD;
    VolumeFlags: DWORD;
    VolumeSerialNumber: DWORD;
  begin
    GetVolumeInformation(PChar(DriveLetter + ':\'),
      nil, MAX_PATH + 1, @VolumeSerialNumber, NotUsed,
      VolumeFlags, nil, 0);
    Result := Format('%8.8X', [VolumeSerialNumber])
  end;

  function GetWindowsIDMod: string;
  begin
    Result := EncryptID(GetWindowsID);
  end;

begin
  {$IFDEF SITE_LICENSES}
  if LicenseType = ltSiteLicense then
  begin
    Result := FLicense;
    Exit;
  end;
  {$ENDIF}
  try
    case LicenseOptions.LicenseSource of
      lsUserDefined: Result := FLicense;
      lsWindowsID: Result := GetWindowsID;
      lsWindowsIDMod: Result := GetWindowsIDMod;
      lsHardDriveSN: Result := GetHardDiskSerial('c');
    end;
    Result := Copy(Result, 1, MaxChars);
  except
    Result := 'Error';
  end;
end;

function TRegwareII.GetRegistered : Boolean;
begin
  Result := CheckRegistered;
end;

function TRegwareII.GetSeed: Int64;
begin
  {$IFDEF HIDE_SEED}
    Result := FSeed xor SEED_STORAGE;  // This is to encrypt FSeed to hide from hackers
  {$ELSE}
    Result := FSeed;
  {$ENDIF}
end;

{ Calculates the registration code to register program }
function TRegwareII.CalculateCode(LicenseName: string): string;
var
  i, Code: integer;
  CodeStr: string;
  Num, SeedValue, Confusion1, Confusion2: Int64;
begin
  // You can modify this algorithm if you want to...
  if (Length(LicenseName) > FMaxChars) or (Length(LicenseName) < FMinChars) then
  begin
    Result := '';  // LicenseName did not fit length requirements
    Exit;
  end;
  SeedValue := FSeed;

  // Spaghetti code to confuse hackers
  Confusion1 := SeedValue + 3;
  Dec(Confusion1, 10);
  Confusion2 := Trunc(SPAGHETTI_1 / Confusion1 + 15);
  Confusion1 := Confusion2 or $1F2E3D4C5B6A7089;
  Inc(Confusion1, 12);
  Confusion2 := Confusion1 and SPAGHETTI_2;

  {$IFDEF PERL_SEED}
  // Make FSeed smaller than 1x10^12 for compatibility with Perl script
  while SeedValue > 1000000000000 do SeedValue := SeedValue shr 1;
  {$ENDIF}

  // More spaghetti code
  i := integer(Confusion2 shr 32);
  Confusion1 := i;
  Confusion1 := (Confusion1 * 0) + 1;  // Should make Confusion1 := 1
  SeedValue := SeedValue * Confusion1;

  i := Length(LicenseName);
  // Takes the last 6 characters (going backwards), converting each
  // character to ASCII and taking the remainder of dividing it into Seed
  while (i >= Length(LicenseName) - 6) and (i > 0) do
  begin
    CodeStr := CodeStr + IntToStr(SeedValue mod Ord(LicenseName[i]));
    Dec(i);
  end;
  Val(CodeStr, Num, Code);
  if FRegCodeSize <= 0 then
    Result := IntToHex(Num, 0)
  else begin
    Result := IntToHex(Num, FRegCodeSize);
    Delete(Result, FRegCodeSize + 1, Length(Result) - FRegCodeSize);
  end;
end;

function TRegwareII.CheckForWinNT: boolean;
var
  VersionInfo: TOsVersionInfo;
begin
  Result := false;
  VersionInfo.dwOSVersionInfoSize := SizeOf(VersionInfo);
  GetVersionEx(VersionInfo);
  if VersionInfo.dwPlatformId = VER_PLATFORM_WIN32_NT then
    if VersionInfo.dwMajorVersion >= 5 then Result := true;  // This is Win NT/2K/XP
end;

function TRegwareII.CheckCodeBanned(RegCode: string): boolean;
var
  i: integer;
begin
  Result := false;
  for i := 0 to FBanList.Count - 1 do
    if CompareText(RegCode, FBanList[i]) = 0 then
    begin
      Result := true;
      break;
    end;
end;

{ Checks if time has expired }
function TRegwareII.CheckExpired: boolean;
begin
  if GetDaysLeft = 0 then Result := true else Result := false;
end;

{ Checks if program is registered }
function TRegwareII.CheckRegistered: boolean;
begin
  if (FRegCode <> '') and (LicenseID <> '') then
  begin
    if (CompareText( FRegCode, CalculateCode(LicenseID) ) = 0) and
       (CheckCodeBanned(FRegCode) = false)  then  // Also check if code is on the ban list
    begin
      Result := true;  // Reg Values check... program registered
    end else
    begin
      Result := false;  // Reg Values invalid... program not registered
      ClearLicenseInfo;
    end;
  end else Result := false;  // Values not there... program not registered
end;

procedure TRegwareII.SetMaxChars(MaxChars: integer);
begin
  if (MaxChars > 100) or (MaxChars < MinChars) then
    raise Exception.Create('Please enter a number between ' +
      IntToStr(FMinChars) + ' and 100')
  else
    FMaxChars := MaxChars;
end;

constructor TRegwareII.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {$IFDEF PRO_VERSION}
  FLicenseOptions := TLicenseOptions.Create;
  {INDY7 - This comment is here for WAK Productions reference only}
  {$ENDIF}
  FBanList := TStringList.Create;

  { Set default values }
  AutoInitialize;
  FRegCodeSize := 12;
  FRegVersion := TREGWARE_VERSION;
end;

destructor TRegwareII.Destroy;
begin
  AutoDestroy;
  FBanList.Free;
  {$IFDEF PRO_VERSION}
  FLicenseOptions.Free;
  {INDY8 - This comment is here for WAK Productions reference only }
  {$ENDIF}
  inherited Destroy;
end;

function TRegwareII.DoRegistration(drLicenseID, drOrganization, drRegCode: string): boolean;
begin
  // This function is here for backwards compatibility.  The old DoRegistration()
  // did not accept an Email field.
  Result := DoRegistration(drLicenseID, drOrganization, drRegCode, '');
end;

function TRegwareII.DoRegistration(drLicenseID, drOrganization, drRegCode, drEmail: string): boolean;
var
  Tick: Cardinal;
begin
  // Delay 2 seconds to prevent brute-force cracking
  // Tick acts as a double-check in case a cracker tries bypassing the
  // sleep function.
  repeat
    Tick := GetTickCount;
    Sleep(2000);

  // Get the system generated LicenseID for LicenseName if not registering
  // a site license, or not a user-defined LicenseID
  //
  {$IFDEF SITE_LICENSES}
  if (LicenseOptions.LicenseType <> ltSiteLicense) and (LicenseOptions.LicenseSource <> lsUserDefined) then
    drLicenseID := LicenseID;
  {$ELSE}
  if LicenseOptions.LicenseSource <> lsUserDefined then
    drLicenseID := LicenseID;
  {$ENDIF}

  until GetTickCount >= Tick + 1999;


  if (Length(drLicenseID) > FMaxChars) or (Length(drLicenseID) < FMinChars) or (Length(drRegCode) = 0) then
  begin  // LicenseID is the incorrect size
    Result := false;
    Exit;
  end;

  drRegCode := UpperCase(drRegCode);  // Registration codes are not case sensitive
  if CompareText(CalculateCode(drLicenseID), drRegCode) <> 0 then Result := false
  else begin  // LicenseID is correct for the entered registration code

    {$IFDEF PRO_VERSION}
    {INDY9 - This comment is here for WAK Productions reference only }
    {$ENDIF}

    FLicense := drLicenseID;
    FOrganization := drOrganization;
    FRegCode := drRegCode;
    FEmail := drEmail;
    SaveRegistryValues;
    Result := true;
  end;
end;

procedure TRegwareII.SetUnregistered;
begin
  ClearLicenseInfo;
  if Timebomb then
  begin
    FExpireTime := Now + FDays;
    FLastCountDown := FDays;
  end else
  begin
    FExpireTime := 0;
    FLastCountDown := FDays;
  end;
  SaveRegistryValues;
end;

procedure TRegwareII.Expire;
begin
  if Assigned(FOnExpire) then FOnExpire(Self);
end;

procedure TRegwareII.ShowNag;
begin
  if Assigned(FOnNagScreen) then FOnNagScreen(Self);
end;

procedure TRegwareII.Loaded;
begin
  inherited;
  CheckVariablesSet;
  LoadRegistryValues;
  if (not Registered) and (CheckClockTampered) then
    ClockChange   // Signal event that the user has tried to change the clock (GetDaysLeft will automatically report that the program has expired)
  else if not Registered then
  begin
    ShowNag;
    SaveRegistryValues;  // We need to resave for the clock-tamper feature
    if CheckExpired = true then
      Expire;
  end;
end;

procedure TRegwareII.SetBanList(Value: TStringList);
begin
  FBanList.Assign(Value);
end;

procedure TRegwareII.WndProc(var Msg: TMessage);
begin
  { It is necessary to save the registry values on the program's termination
    so that it can be determined later if the user set his clock backwards via
    the LastCountDown property

    This cannot be done in the deconstructor because Delphi will execute
    the deconstructor when the project closes.  This will cause the registry
    values to be erased with blank material. }
  if Msg.Msg = WM_QUIT then SaveRegistryValues;
end;

procedure TRegwareII.ClockChange;
begin
  if Assigned(FOnClockChange) then FOnClockChange(Self);
end;

function TRegwareII.CheckClockTampered: boolean;
var
  DateDifference: integer;  // Difference between expiration date and now
  FileCreateTime: integer;
begin
  {$DEFINE CHECK_CREATETIME_TAMPER}
  Result := false;
  if FCheckTamper = true then
  begin
    DateDifference := Trunc(FExpireTime - Date);
    if DateDifference > FLastCountDown then // User set clock backwards
      Result := true;

    {$IFDEF CHECK_CREATETIME_TAMPER}
    try
      // Also check the file's creation time to detect possible tampering
      FileCreateTime := FileAge(Application.ExeName);
      if FileCreateTime <> -1 then
      begin
        //GetFileTime(FileHandle, @FileCreateTime, nil, nil);
        if (Date - Trunc(FileDateToDateTime(FileCreateTime)) < 0) then
          // Current system time is before file creation time
          Result := true;
      end;
    except
      // Don't do anything
    end;
    {$ENDIF}
  end;
end;

function TRegwareII.GetExpired: boolean;
begin
  Result := CheckExpired;
end;

procedure TRegwareII.SetSeed(Value: Int64);
begin
  {$IFDEF HIDE_SEED}
  if (Value > 1000) then
    FSeed := Value xor SEED_STORAGE;  // This is to encrypt FSeed to hide from hackers
  {$ELSE}
  if (Value > 1000) then
    FSeed := Value;
  {$ENDIF}
end;


procedure TRegwareII.SaveRegistryValues;
var
  RegData: TRegInfo;
  Registry: TRegistry;
begin
  if ValidateGUID(FProgGUID) = false then Exit;

  Registry := TRegistry.Create(KEY_WRITE);
  try
    Registry.Lazywrite := false;
    if CheckForWinNT = true then
    begin
      Registry.RootKey := HKEY_CLASSES_ROOT;
      Registry.OpenKey('\CLSID\' + FProgGUID + '\Info', true);
    end else
    begin
      Registry.RootKey := HKEY_LOCAL_MACHINE;
      Registry.OpenKey('\Software\CLASSES\CLSID\' + FProgGUID + '\Info', true);
    end;
    with RegData do
    begin
      License := FLicense;
      Organization := FOrganization;
      RegCode := FRegCode;
      Exp1 := FEmail;
      RegVersion := TREGWARE_VERSION;

      if not Registered then
      begin
        Exp2 := 0;  // Not registered, so do not specify site license or single-user

        ExpireTime := FExpireTime;
        if CheckClockTampered = false then
          LastCountDown := Trunc(ExpireTime - Date)
        else
          LastCountDown := FLastCountDown;   // Only save this value if clock is correct
      end else
      begin
        // Set whether we are using a site license or single-user license
        if LicenseOptions.LicenseType = ltSiteLicense then
          Exp2 := SITE_LICENSE
        else
          Exp2 := SINGLE_USER_LICENSE;

        ExpireTime := 0;
        LastCountDown := 0;
      end; // if..then..else

    end; // with RegData do
      Registry.WriteBinaryData('Data', RegData, SizeOf(RegData));
      Registry.CloseKey;
  finally
    Registry.Free;
  end; // try..finally
end;

procedure TRegwareII.ClearLicenseInfo;
begin
  // Clear the false values
  FLicense := '';
  FOrganization := '';
  FRegCode := '';
end;

procedure TRegwareII.SetMinChars(MinChars: integer);
begin
  if (MinChars > FMaxChars) or (MinChars < 3) then
    raise Exception.Create('Please enter a number between 3 and ' +
      IntToStr(FMaxChars));
end;

procedure TRegwareII.LoadRegistryValues;
var
  RegData: TRegInfo;
  Registry: TRegistry;
begin
  if ValidateGUID(FProgGUID) = false then Exit;

  Registry := TRegistry.Create(KEY_READ);
  try
    Registry.Lazywrite := false;
    if CheckForWinNT = true then
    begin
      Registry.RootKey := HKEY_CLASSES_ROOT;
      Registry.OpenKeyReadOnly('\CLSID\' + FProgGUID + '\Info');
    end else
    begin
      Registry.RootKey := HKEY_LOCAL_MACHINE;
      Registry.OpenKeyReadOnly('\Software\CLASSES\CLSID\' + FProgGUID + '\Info');
    end;
    if Registry.ValueExists('Data') then
    begin
      Registry.ReadBinaryData('Data', RegData, SizeOf(RegData));
      with RegData do
      begin
        FLicense := License;
        FOrganization := Organization;
        FRegCode := RegCode;
        FEmail := Exp1;
        FExpireTime := ExpireTime;
        FRegVersion := RegVersion;
        FLastCountDown := LastCountDown;

        if Exp2 = SITE_LICENSE then
          LicenseOptions.LicenseType := ltSiteLicense
        else if Exp2 = SINGLE_USER_LICENSE then
          LicenseOptions.LicenseType := ltSingleUserLicense
        else
        begin
          LicenseOptions.LicenseType := ltSingleUserLicense;  // Since it is neither value, that may hint some tampering with the value
          ClearLicenseInfo;
        end;
      end;
    end else
    begin
      SetUnregistered;
    end;
    Registry.CloseKey;
  finally
    Registry.Free;
  end; // try..finally
end;

procedure TRegwareII.CheckVariablesSet;
begin
  if ValidateGUID(FProgGUID) = false then
    ShowMessage('Mr. Programmer forgot to set the ProgGUID property to a ' +
      'valid GUID string!  Tsk, Tsk...');
  if FSeed = 0 then
    ShowMessage('Mr. Programmer forgot to set the Seed property to a ' +
      'valid integer between 1000 and 2^63!  Tsk, Tsk...');
end;



end.


